/****************************************************************************

  (c) SYSTEC electronic GmbH, D-07973 Greiz, August-Bebel-Str. 29
      www.systec-electronic.de

  Project:      COMBImodul-C14

  Description:  source file for io driver api function 

  -------------------------------------------------------------------------

                $RCSfile: C14Pwm.c,v $
                
                $Author: A.Suess $
                
                $Revision: 1.1.1.1 $  $Date: 2004/09/16 11:56:02 $
                
                $State: Exp $
                
                Build Environment:
                KEIL uVision 2

  -------------------------------------------------------------------------

  Revision History:

  2004/03/29 -as:   start of the implementation, version 1.00

  2004/06/18 -as:   multiple segments supported for PTO

****************************************************************************/

#include "generic/global.h"
#include "generic/gmacros.h"

#include "../include/C14drv.h"
#include "../include/C14def.h"


/***************************************************************************/
/*                                                                         */
/*                                                                         */
/*          G L O B A L   D E F I N I T I O N S                            */
/*                                                                         */
/*                                                                         */
/***************************************************************************/

//---------------------------------------------------------------------------
// const defines
//---------------------------------------------------------------------------

#define POUT_CHN0 CC2_OUT_CC0IO
#define POUT_CHN1 CC2_OUT_CC1IO

#define CHANNEL_ACTIV   bitPWMPortValueOne_g
#define CHANNEL_INACTIV bitPWMPortValueZero_g

//---------------------------------------------------------------------------
// local types
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
// modul globale vars
//---------------------------------------------------------------------------
extern bit bitPWMPortValueOne_g;
extern bit bitPWMPortValueZero_g;

//---------------------------------------------------------------------------
// local function prototypes
//---------------------------------------------------------------------------

/***************************************************************************/
/*                                                                         */
/*                                                                         */
/*          C L A S S  <C14 PWM/PTO API>                                   */
/*                                                                         */
/*                                                                         */
/***************************************************************************/
//
// Description:
// 
/***************************************************************************/


//=========================================================================//
//                                                                         //
//          P R I V A T E   D E F I N I T I O N S                          //
//                                                                         //
//=========================================================================//

//---------------------------------------------------------------------------
// const defines
//---------------------------------------------------------------------------
#define PWM_TICKS_PER_MSEC      1250

#define INT_PRIORITY_PWM0       0x0034      // ILVL = 13, GLVL = 0
#define INT_PRIORITY_PWM1       0x0035      // ILVL = 13, GLVL = 1

#define MASK_CHN_0              0x0001
#define MASK_CHN_1              0x0002
#define MASK_CHN_2              0x0004
#define MASK_CHN_3              0x0008

// control register for PWM/PTO unit
// Bit 0: selection base clock channel P0: 0 --> 800ns, 1 --> 1ms
// Bit 1: selection base clock channel P1: 0 --> 800ns, 1 --> 1ms

#define BASE_CLOCK_MSEC         0x0001      

#define OVERRUN_0               0x0001
#define ABORT_0                 0x0002
#define DELTA_ERR_0             0x0004
#define OVERRUN_1               0x0010
#define ABORT_1                 0x0020
#define DELTA_ERR_1             0x0040

//---------------------------------------------------------------------------
// local types
//---------------------------------------------------------------------------
typedef struct 
{
    WORD  m_wPeriode;       // start value of periode time
    WORD  m_iDelta;         // difference of periode time between two impulses
    DWORD m_dwImpulse;      // number of pulses per segment (same delta)

} tPtoEntry;


//---------------------------------------------------------------------------
// local vars
//---------------------------------------------------------------------------
static WORD      sdata wMode_l;
static WORD      sdata wError_l;

static WORD      sdata wPWM0Pulse_l;
static WORD      sdata wPWM1Pulse_l;
static WORD      sdata wPWM0Periode_l;
static WORD      sdata wPWM1Periode_l;
static WORD      sdata wBase0ImpulseCounter_l;
static WORD      sdata wBase1ImpulseCounter_l;

// vars of PTO generator
static int       sdata iPTO0Delta_l;
static int       sdata iPTO1Delta_l;
static DWORD     sdata dwPTO0ImpulseCounter_l;
static DWORD     sdata dwPTO1ImpulseCounter_l;
static tPtoEntry aPto0Profile_l[255];
static WORD      sdata wPto0ProfileWrIdx_l;
static WORD      sdata wPto0ProfileRdIdx_l;

static tPtoEntry aPto1Profile_l[255];
static WORD      sdata wPto1ProfileWrIdx_l;
static WORD      sdata wPto1ProfileRdIdx_l;

// pointer to event handler
static void   (* fpC14HandlerCC24_l)(void);
static void   (* fpC14HandlerCC25_l)(void);
static void   (* fpC14HandlerCC26_l)(void);
static void   (* fpC14HandlerCC27_l)(void);

//---------------------------------------------------------------------------
// local function prototypes
//---------------------------------------------------------------------------

static void C14Pwm0HandlerCC24 (void);
static void C14Pwm1HandlerCC25 (void);
static void C14Pwm0HandlerCC26 (void);
static void C14Pwm1HandlerCC27 (void);
static void C14Pto0HandlerCC24 (void);
static void C14Pto1HandlerCC25 (void);
static void C14Pto0HandlerCC26 (void);
static void C14Pto1HandlerCC27 (void);
static BYTE C14IsrSetCC24      (BYTE bState_p, void (*fpIsrHandler_p)(void));
static BYTE C14IsrSetCC25      (BYTE bState_p, void (*fpIsrHandler_p)(void));
static BYTE C14IsrSetCC26      (BYTE bState_p, void (*fpIsrHandler_p)(void));
static BYTE C14IsrSetCC27      (BYTE bState_p, void (*fpIsrHandler_p)(void));


//=========================================================================//
//                                                                         //
//          P U B L I C   F U N C T I O N S                                //
//                                                                         //
//=========================================================================//


//---------------------------------------------------------------------------
//
// Function:     C14PwmInit
//
// Description:  This function init. the PWM-Timer and reset the PWM
//               channels. The PWM outputs remains in inactiv state.
//               The PWM channel are initialized but not started. 
//
// Parameters:   none
//
// 
// Returns:      none
//
//
// State:        tested (18.06.2004 A.S.)
//
//---------------------------------------------------------------------------

void PUBLIC C14PwmInit (void)
{
    //  -----------------------------------------------------------------------
    //  Configuration of PWM channel register
    //  -----------------------------------------------------------------------
    wMode_l        = 0;  // all channels switch off
    wError_l       = 0;  // clear error register

    wPWM0Pulse_l   = 0;
    wPWM0Periode_l = 0;
    wPWM1Pulse_l   = 0;
    wPWM1Periode_l = 0;
    wBase0ImpulseCounter_l = -1;
    wBase1ImpulseCounter_l = -1;

    iPTO0Delta_l = 0;
    iPTO1Delta_l = 0;
    dwPTO0ImpulseCounter_l = 0L;
    dwPTO1ImpulseCounter_l = 0L;
    wPto0ProfileWrIdx_l = 0;
    wPto0ProfileRdIdx_l = 0;
    wPto1ProfileWrIdx_l = 0;
    wPto1ProfileRdIdx_l = 0;

    //  -----------------------------------------------------------------------
    //  Configuration of CAPCOM2 Control:
    //  -----------------------------------------------------------------------
    //  - the contents of the port register is visble at output port
    //  - non-staggered mode is enabled
    CC2_IOC        =  0x0004;
    CC2_M4         =  0x0000; // P0/1: T7
    CC2_M6         =  0x0000; // P0/1: T7
    CC2_DRM        =  0x0005; // Double-Register-Mode for CC16 and CC17 disabled
    CC2_CC16IC     =  0;
    CC2_CC17IC     =  0;
    CC2_CC24IC     =  0;
    CC2_CC25IC     =  0;
    
    //  -----------------------------------------------------------------------
    //  Configuration of CAPCOM2 Timer 7 and Timer 8:
    //  -----------------------------------------------------------------------
    //  The timer defines the periode time of PWM signal. The timer starts with the
    //  value of reload register every overflow and counting up. The reload value 
    //  defines the max. resolution (8Bit/10Bit/12Bit/16Bit) of the channel.
    //  start value related to resolution x: treload = 0x10000 - 2^x
    //  periode time: t = ttimer * (0x10000- treload)
    //  
    //  - timer 7 works in timer mode, used for PWM0, 1, prescaler = 32
    //  - timer 8 works in timer mode, used for PWM2, 3, prescaler = 0
    CC2_T78CON     =  0x0005;
    CC2_T7         =  0;
    CC2_T7REL      =  0;
    CC2_T8         =  0x0000;
    CC2_T8REL      =  0x0000;
    CC2_T7IC       =  0;
    CC2_T8IC       =  0;
    CC2_T78CON_T7R =  1;
    CC2_T78CON_T8R =  1;
}

//---------------------------------------------------------------------------
//
// Function:     C14PwmState
//
// Description:  Function gets the state of PWM channel.
//
// Parameters:   bChannel_p:     index of channel
//               pointer to channel state
// 
// Returns:      channel state
//               Bit0: BUSY-Flag
//               Bit1: Overrun
//               Bit2: Abort
//               Bit3: Delta Error
//               error code
//
//
// State:        tested (18.06.2004 A.S.)
//
//---------------------------------------------------------------------------

BYTE PUBLIC C14PwmState(BYTE bChannel_p, BYTE *pbState_p)
{
    switch (bChannel_p)
    {
        // -----------------------------------------------
        case PWM0: *pbState_p = (BYTE)((wError_l & 0x0F) | ((ALTSEL1P9 & MASK_CHN_0)?CHN_BUSY:0));
        break;
                   
        // -----------------------------------------------
        case PWM1: *pbState_p = (BYTE)(((wError_l >> 4) & 0x0F) | ((ALTSEL1P9 & MASK_CHN_1)?CHN_BUSY:0));
        break;
                   
        // -----------------------------------------------
        case PWM2: *pbState_p = (ALTSEL1P9 & MASK_CHN_2)?CHN_BUSY:0;
        break;
                   
        // -----------------------------------------------
        case PWM3: *pbState_p = (ALTSEL1P9 & MASK_CHN_3)?CHN_BUSY:0;
        break;

        // -----------------------------------------------
        default:
             return (C14_INVALID_CHANNEL);
        break;
    }
    return (C14_SUCCESSFUL);
}

//---------------------------------------------------------------------------
//
// Function:     C14PwmPtoTimer
//
// Description:  The function sets the base clock for PWM/PTO channels.
//
//
// Parameters:   bChannel_p:    index of channel
//               bScale_p:      base clock resolution
//                              0x00: 25ns
//                              0x01: 50ns
//                              0x02:100ns
//                              0x03:200ns
//                              0x04:400ns
//                              0x05:800ns
//                              0x06:1.6s
//                              0x07:3.2s
//                              ...
//                              0x0F:1ms
//
// 
// Returns:      errorcode
//
//
// State:        tested (18.06.2004 A.S.)
//
//---------------------------------------------------------------------------

BYTE PUBLIC C14PwmPtoTimer (BYTE bChannel_p, BYTE bScale_p)
{
WORD wMask;

    bScale_p &= 0x0F;

    switch (bChannel_p)
    {
        // -----------------------------------------------
        // For channel  P0 and P1 are only two base clocks 
        // supported: 1ms und 800ns. 
        case PWM0: 
        case PWM1: 
             wMask = BASE_CLOCK_MSEC;// create bit mask for channel P0
             if (bChannel_p == PWM1)
                 wMask     <<= 1; // create bit mask for channel P1 

             // store configuration for later use
             if (bScale_p == 0x0F)
                 wMode_l |= wMask;  // = 1 --> 1ms base clock
             else
                 wMode_l &= ~wMask; // = 0 --> 800ns base clock
        break;
                   
        // -----------------------------------------------
        case PWM2: 
        case PWM3: 
             CC2_T78CON_T8R  = 0;
             CC2_T8 = CC2_T8REL = 0;
             CC2_T78CON     &= 0xF8FF;
             CC2_T78CON     |= (bScale_p << 8);
             CC2_T78CON_T8R  = 1;
        break;
                   
        default:
             return (C14_INVALID_CHANNEL);
        break;
    }

    return (C14_SUCCESSFUL);
}

//---------------------------------------------------------------------------
//
// Function:     C14PwmParam
//
// Description:  The function sets the pulse time and periode time.
//               For channel P0 and P1 the pulse time is set as number 
//               of base clocks. 
//               Changing periode time is done asynchron related to running cycle.
//               The channel is stopped. The pulse time must set again and the
//               the channel is restarted.
//               For channel P2 and P3 the pulse time is set as relative
//               on-time (16Bit value) and internal scaled to the selected 
//               periode time. The value 0 means 0% on-time, the value
//               0xFFFF means 100% on-time.
//
//
// Parameters:   bChannel_p:    index of channel
//               wPeriode_p:    periode time
//               wPulse_p:      P0/1: pulse time
//                              P2/3: relative on-time in % (16Bit Auflsung)
//
// 
// Returns:      errorcode
//
//
// State:        tested (18.06.2004 A.S.)
//
//---------------------------------------------------------------------------

BYTE PUBLIC C14PwmParam(BYTE bChannel_p, WORD wPeriode_p, WORD wPulse_p)
{
WORD  wPulse;
bit   fSignal = FALSE;

    switch (bChannel_p)
    {
        // -----------------------------------------------
        // If the last value of pulse time was 0% or 100% 
        // then is not known, how long was this state activ.
        // Therefor the ISR will be triggered to re-start a
        // new cycle with the new values. 
        // For the base clock of 800ns the periode time is
        // set to a min. value of 100s = 125 clocks a 800ns. 
        case PWM0: 
             // check up the lowest value of periode time
             // for base clock = 1ms
             if (wMode_l & BASE_CLOCK_MSEC)
             {
                if (wPeriode_p < 2)
                   wPeriode_p = 2;
             }
             else
             {  // check up the lowest value of periode time
                // for base clock of 800ns
                if (wPeriode_p < 125)
                   wPeriode_p = 125;
             }

             // Retrigger ISR if ... 
             // - changed value 
             // - last on-time 0% or 100%
             // - channel not activ
             if ( (wPWM0Periode_l != wPeriode_p)    || 
                  (wPWM0Pulse_l   == 0)             || 
                  (wPWM0Pulse_l   >= wPWM0Periode_l)||
                  !(ALTSEL1P9 & MASK_CHN_0) )
                fSignal      = TRUE;

             if (wMode_l & BASE_CLOCK_MSEC)
             {
                // stop timer T7 --> disable ISR
                CC2_T78CON_T7R = 0;
                wPWM0Periode_l = wPeriode_p;
                wPWM0Pulse_l   = wPulse_p;
                if (fSignal)
                {  // retrigger ISR requested
                   wBase0ImpulseCounter_l = -1;
                   CC2_CC26IC_IR = 1;
                }
                CC2_T78CON_T7R = 1;
             }
             else
             {
                CC2_T78CON_T7R = 0;
                wPWM0Periode_l = wPeriode_p;
                wPWM0Pulse_l   = wPulse_p;
                CC2_CC24IC_IR  = fSignal;
                CC2_T78CON_T7R = 1;
             }
        break;
                   
        // -----------------------------------------------
        case PWM1: 
             // check up the lowest value of periode time
             // for base clock = 1ms
             if (wMode_l & (BASE_CLOCK_MSEC << 1))
             {
                if (wPeriode_p < 2)
                    wPeriode_p = 2;
             }
             else
             {  // check up the lowest value of periode time
                // for base clock of 800ns
                if (wPeriode_p < 125)
                    wPeriode_p = 125;
             }

             // Retrigger ISR if ... 
             // - changed value 
             // - last on-time 0% or 100%
             // - channel not activ
             if ( (wPWM1Periode_l != wPeriode_p)    || 
                  (wPWM1Pulse_l   == 0)             || 
                  (wPWM1Pulse_l   >= wPWM1Periode_l)||
                  !(ALTSEL1P9 & MASK_CHN_1) )
                fSignal      = TRUE;

             if (wMode_l & (BASE_CLOCK_MSEC<<1))
             {
                // stop timer T7 --> disable ISR
                CC2_T78CON_T7R = 0;
                wPWM1Periode_l = wPeriode_p;
                wPWM1Pulse_l   = wPulse_p;
                if (fSignal)
                {
                   // retrigger ISR requested
                   wBase1ImpulseCounter_l = -1;
                   CC2_CC27IC_IR = 1;
                }
                CC2_T78CON_T7R = 1;
             }
             else
             {
                CC2_T78CON_T7R = 0;
                wPWM1Periode_l = wPeriode_p;
                wPWM1Pulse_l   = wPulse_p;
                CC2_CC25IC_IR  = fSignal;
                CC2_T78CON_T7R = 1;
             }
        break;

        // -----------------------------------------------
        case PWM2:
        case PWM3: 
             // reload periode time if changed
             if (CC2_T8REL != (WORD)(0x10000L - (DWORD)wPeriode_p))
             {
                // stop base timer to reload new values
                CC2_T78CON_T8R = 0;
                CC2_T8 = CC2_T8REL = (WORD)(0x10000L - (DWORD)wPeriode_p);
                // start PWM signal
                CC2_T78CON_T8R = 1;
             }
             else
             {
                if (bChannel_p == PWM2)
                {
                   wPulse    = (WORD)( ((0x10000L - (DWORD)CC2_T8REL) * (DWORD)wPulse_p) >> 16);
                   CC2_CC18  = (WORD)(0x10000L - (DWORD)wPulse);
                }
                else
                {
                   wPulse    = (WORD)( ((0x10000L - (DWORD)CC2_T8REL) * (DWORD)wPulse_p) >> 16);
                   CC2_CC19  = (WORD)(0x10000L - (DWORD)wPulse);
                }
             }
        break;

        // -----------------------------------------------
        default:
             return (C14_INVALID_CHANNEL);
        break;
                   
    }

    return (C14_SUCCESSFUL);
}

//---------------------------------------------------------------------------
//
// Function:     C14PwmEnable
//
// Description:  The function starts/stops the selected PWM channel.
//               The alternative function of port pin are enabled.
//               A continoues signal is generated until the channel 
//               is disabled by this function. 
//               The PWM generator must start only from inactiv state.
//
// Parameters:   bChannel_p:       index of channel
//               bitRun:           TRUE  -> PWM generator starts 
//                                 FALSE -> PWM generator stops
//
// 
// Returns:      errorcode
//
//
// State:        tested (18.06.2004 A.S.)
//
//---------------------------------------------------------------------------

BYTE PUBLIC C14PwmEnable(BYTE bChannel_p, bit bitRun_p)
{
    switch (bChannel_p)
    {
        // -----------------------------------------------
        case PWM0: 
             if (bitRun_p)
             {
                //------------------------------------------------
                // set start value of port pin
                POUT_CHN0 = CHANNEL_INACTIV;
                // PWM line is switched by related CCx-channel
                // or by setting the related bit in register CC2_OUT. 
                // For that purpose the alternative function has to enable.
                ALTSEL1P9 |= MASK_CHN_0;
                //------------------------------------------------
                // According to the resolution of base clock the
                // PWM line is controlled by COMPARE unit or
                // by timer interrupt.
                if (wMode_l & BASE_CLOCK_MSEC)
                   // base clock 1ms
                   C14IsrSetCC26 (TRUE, C14Pwm0HandlerCC26);
                else
                   // base clock 800ns
                   C14IsrSetCC24 (TRUE, C14Pwm0HandlerCC24);
             }
             else
             {
                // disable alternative function -> controlling of PWM line
                // by process image now
                ALTSEL1P9 &= ~MASK_CHN_0;
                POUT_CHN0 = CHANNEL_INACTIV;
                // disable interrupts, CC16, CC24, CC26 stop
                C14IsrSetCC26 (FALSE, NULL);
                C14IsrSetCC24 (FALSE, NULL);
                // reset flags
                wError_l &= ~(OVERRUN_0 | ABORT_0 | DELTA_ERR_0);
             }
        break;
                   
        // -----------------------------------------------
        case PWM1: 
             if (bitRun_p)
             {
                //------------------------------------------------
                // set start value of port pin
                POUT_CHN1 = CHANNEL_INACTIV;
                // PWM line is switched by related CCx-channel
                // or by setting the related bit in register CC2_OUT. 
                // For that purpose the alternative function has to enable.
                ALTSEL1P9 |= MASK_CHN_1;
                //------------------------------------------------
                // According to the resolution of base clock the
                // PWM line is controlled by COMPARE unit or
                // by timer interrupt.
                if (wMode_l & (BASE_CLOCK_MSEC << 1))
                   // base clock 1ms
                   C14IsrSetCC27 (TRUE, C14Pwm1HandlerCC27);
                //---------------------------------------------------------
                else
                   // base clock 800ns
                   C14IsrSetCC25 (TRUE, C14Pwm1HandlerCC25);
             }
             else
             {
                // disable alternative function -> controlling of PWM line
                // by process image now
                ALTSEL1P9 &= ~MASK_CHN_1;
                // Impulsdauer abgelaufen --> Kanal ausschalten
                POUT_CHN1 = CHANNEL_INACTIV;
                // disable interrupts, CC17, CC25, CC27 stop
                C14IsrSetCC27 (FALSE, NULL);
                C14IsrSetCC25 (FALSE, NULL);
                // reset flags
                wError_l &= ~(OVERRUN_1 | ABORT_1 | DELTA_ERR_1);
             }
        break;

        // -----------------------------------------------
        case PWM2: 
             if (bitRun_p)
             {
                // alternative function "COMPARE OUTPUT" enable
                ALTSEL1P9 |= MASK_CHN_2;
                // CC18: Compare Mode 3, T8
                CC2_M4 |= 0x0F00;  // start
             }
             else
             {
                CC2_M4 &= 0xF0FF;  // stop
                ALTSEL1P9 &= ~MASK_CHN_2;
             }
        break;
                   
        // -----------------------------------------------
        case PWM3: 
             if (bitRun_p)
             {
                // alternative function "COMPARE OUTPUT" enable
                ALTSEL1P9 |= MASK_CHN_3;
                // CC19: Compare Mode 3, T8
                CC2_M4 |= 0xF000;  // start
             }
             else
             {
                CC2_M4 &= 0x0FFF;  // stop
                ALTSEL1P9 &= ~MASK_CHN_3;
             }
        break;

        // -----------------------------------------------
        default:
             return (C14_INVALID_CHANNEL);
        break;
                   
    }

    return (C14_SUCCESSFUL);
}

//---------------------------------------------------------------------------
//
// Function:     C14Pwm0HandlerCC24
//
// Description:  ISR-function for PWM-channel P0 and base clock of 800ns
//               This interrupt handler is called at the end of a PWM periode.
//
//               Channel CC24 controls the periode time and
//               CC16 the pulse time. The PWM line starts with 
//               an activ level and switchs to inactiv state at 
//               the end of pulse time. After periode time is 
//               over this interrupt will be called to start the 
//               next periode. 
// 
// Parameters:   none 
//
// 
// Returns:      none
//
//
// State:        tested (18.06.2004 A.S.)
//
//---------------------------------------------------------------------------

static void C14Pwm0HandlerCC24 (void)
{
    // on-time 0%
    if (wPWM0Pulse_l == 0)
    {
       POUT_CHN0 = CHANNEL_INACTIV;
       CC2_T78CON_T7R = 1;
       return;
    }

    // on-time 100%
    if (wPWM0Pulse_l >= wPWM0Periode_l)
    {
       POUT_CHN0 = CHANNEL_ACTIV;
       CC2_T78CON_T7R = 1;
       return;
    }

    // stop timer T7 to re-init the next periode ...
    CC2_T78CON_T7R = 0;
    // ... and start next CC24-periode
    CC2_SEE |=  0x0101;
    POUT_CHN0 = CHANNEL_ACTIV;
    CC2_CC24 = CC2_T7 + wPWM0Periode_l;
    // reload pulse time ...
    CC2_CC16 = CC2_T7 + wPWM0Pulse_l;
    // start periode
    CC2_T78CON_T7R = 1;
}

//---------------------------------------------------------------------------
//
// Function:     C14Pwm1HandlerCC25
//
// Description:  ISR-function for PWM-channel P1 and base clock of 800ns
//               This interrupt handler is called at the end of a PWM periode.
//
//               Channel CC25 controls the periode time and
//               CC17 the pulse time. The PWM line starts with 
//               an activ level and switchs to inactiv state at 
//               the end of pulse time. After periode time is 
//               over this interrupt will be called to start the 
//               next periode. 
// 
// Parameters:   none 
//
// 
// Returns:      none
//
//
// State:        tested (18.06.2004 A.S.)
//
//---------------------------------------------------------------------------

static void C14Pwm1HandlerCC25 (void)
{
    // on-time 0%
    if (wPWM1Pulse_l == 0)
    {
       POUT_CHN1 = CHANNEL_INACTIV;
       CC2_T78CON_T7R = 1;
       return;
    }

    // on-time 100%
    if (wPWM1Pulse_l >= wPWM1Periode_l)
    {
       POUT_CHN1 = CHANNEL_ACTIV;
       CC2_T78CON_T7R = 1;
       return;
    }

    // stop timer T7 to re-init the next periode ...
    CC2_T78CON_T7R = 0;
    // ... and start next CC25-periode
    CC2_SEE |=  0x0202;
    POUT_CHN1 = CHANNEL_ACTIV;
    CC2_CC25 = CC2_T7 + wPWM1Periode_l;
    CC2_CC17 = CC2_T7 + wPWM1Pulse_l;
    // start periode
    CC2_T78CON_T7R = 1;
}

//---------------------------------------------------------------------------
//
// Function:     C14Pwm0HandlerCC26
//
// Description:  ISR-function for PWM-channel P0 and base clock of 1ms
//
//               The PWM line starts with an activ level and switchs 
//               to inactiv state at the end of pulse time. After periode 
//               time is over this interrupt will be called to start the 
//               next periode. 
// 
// Parameters:   none 
//
// 
// Returns:      none
//
//
// State:        tested (18.06.2004 A.S.)
//
//---------------------------------------------------------------------------

static void C14Pwm0HandlerCC26(void)
{
    // reload timer for generation of base clock
    CC2_CC26 = CC2_T7 + PWM_TICKS_PER_MSEC;

    // count the base clocks for controlling PWM line state
    wBase0ImpulseCounter_l++;
    // Number of base clocks to switch PWM line on reached?
    // If on-time 0%, then channel allways off.
    if (wBase0ImpulseCounter_l < wPWM0Pulse_l)
    {
       POUT_CHN0 = CHANNEL_ACTIV;
       return;
    }

    // periode time over?
    // If on-time 100%, then channel allways on.
    if (wBase0ImpulseCounter_l >= wPWM0Periode_l)
    {
       wBase0ImpulseCounter_l = -1;
       // re-trigger ISR to start the next periode
       CC2_CC26IC_IR = 1;
       return;
    }

    // puls time over --> channel switchs off
    POUT_CHN0 = CHANNEL_INACTIV;
}

//---------------------------------------------------------------------------
//
// Function:     C14Pwm1HandlerCC27
//
// Description:  ISR-function for PWM-channel P1 and base clock of 1ms
//
//               The PWM line starts with an activ level and switchs 
//               to inactiv state at the end of pulse time. After periode 
//               time is over this interrupt will be called to start the 
//               next periode. 
//
// 
// Parameters:   none
//
// 
// Returns:      none
//
//
// State:        tested (18.06.2004 A.S.)
//
//---------------------------------------------------------------------------

static void C14Pwm1HandlerCC27(void)
{
    // reload timer for generation of base clock
    CC2_CC27 = CC2_T7 + PWM_TICKS_PER_MSEC;

    // count the base clocks for controlling PWM line state
    wBase1ImpulseCounter_l++;
    // Number of base clocks to switch PWM line on reached?
    // If on-time 0%, then channel allways off.
    if (wBase1ImpulseCounter_l < wPWM1Pulse_l)
    {
       POUT_CHN1 = CHANNEL_ACTIV;
       return;
    }

    // periode time over?
    // If on-time 100%, then channel allways on.
    if (wBase1ImpulseCounter_l >= wPWM1Periode_l)
    {
       wBase1ImpulseCounter_l = -1;
       // re-trigger ISR to start the next periode
       CC2_CC27IC_IR = 1;
       return;
    }

    // puls time over --> channel switchs off
    POUT_CHN1 = CHANNEL_INACTIV;
}


//---------------------------------------------------------------------------
//
// Function:     C14PtoParam
//
// Description:  The function sets the next segment to generate a pulse  
//               profile (PTO). The values are stored in a table.  
//               The extent of validity is checked up. The periode time
//               is limited to a lowest value according to the resolution 
//               of base clock (base clock 1ms: lowest periode time = 2ms
//               base clock 800ns: lowest periode time = 100s).
//               The number of pulses are allways greater than 0.
// 
// Parameters:   index of channel
//               periode time 
//               Delta (difference) periode time
//               number of pulses
//
// 
// Returns:      none
//
//
// State:        tested (18.06.2004 A.S.)
//
//---------------------------------------------------------------------------

BYTE PUBLIC C14PtoParam(BYTE bChannel_p, WORD wPeriode_p, int iDelta_p, DWORD dwImpulse_p)
{
BYTE  bRetCode = C14_SUCCESSFUL;
WORD  wNextBuffWrIdx;

    if (!dwImpulse_p)
       dwImpulse_p = 1L;

    switch (bChannel_p)
    {
        //-------------------------------------------------------
        case PTO0:
             if (wMode_l & BASE_CLOCK_MSEC)
             {
                if (wPeriode_p < 2)
                   wPeriode_p = 2;
             }
             else
             {  
                if (wPeriode_p < 125)
                   wPeriode_p = 125;
             }

             // store parameter to table
             wNextBuffWrIdx = wPto0ProfileWrIdx_l;

             wNextBuffWrIdx += 1;
             if (wNextBuffWrIdx >= tabentries(aPto0Profile_l))
             {
                wNextBuffWrIdx = 0;
             }

             // free buffer availabale (write index and read index not identical)?
             if (wNextBuffWrIdx != wPto0ProfileRdIdx_l)
             {
                // free buffer availabale -> write data to table
                aPto0Profile_l[wPto0ProfileWrIdx_l].m_wPeriode  = wPeriode_p;
                aPto0Profile_l[wPto0ProfileWrIdx_l].m_iDelta    = iDelta_p;
                aPto0Profile_l[wPto0ProfileWrIdx_l].m_dwImpulse = dwImpulse_p;

                wPto0ProfileWrIdx_l = wNextBuffWrIdx;
             }
             else
             {
                // no table entry is free
                bRetCode = C14_BUFFER_OVERRUN;
             }
        break;

        //-------------------------------------------------------
        case PTO1:
             if (wMode_l & (BASE_CLOCK_MSEC <<1))
             {
                if (wPeriode_p < 2)
                   wPeriode_p = 2;
             }
             else
             {  
                if (wPeriode_p < 125)
                   wPeriode_p = 125;
             }

             // store parameter to table
             wNextBuffWrIdx = wPto1ProfileWrIdx_l;

             wNextBuffWrIdx += 1;
             if (wNextBuffWrIdx >= tabentries(aPto1Profile_l))
             {
                wNextBuffWrIdx = 0;
             }

             // free buffer availabale (write index and read index not identical)?
             if (wNextBuffWrIdx != wPto1ProfileRdIdx_l)
             {
                // free buffer availabale -> write data to table
                aPto1Profile_l[wPto1ProfileWrIdx_l].m_wPeriode  = wPeriode_p;
                aPto1Profile_l[wPto1ProfileWrIdx_l].m_iDelta    = iDelta_p;
                aPto1Profile_l[wPto1ProfileWrIdx_l].m_dwImpulse = dwImpulse_p;

                wPto1ProfileWrIdx_l = wNextBuffWrIdx;
             }
             else
             {
                // no table entry is free
                bRetCode = C14_BUFFER_OVERRUN;
             }
        break;

        //-------------------------------------------------------
        default:
             bRetCode = C14_INVALID_CHANNEL;
        break;
    }

    return (bRetCode);
}

//---------------------------------------------------------------------------
//
// Function:     C14PtoEnable
//
// Description:  The function starts/stops a PTO channel. 
//
//               The channel works allways in Single Event Mode.
//               The channel CC24/25 controls periode time of signal.
//               The channel CC16/17 controls pulse time and switchs
//               the PTO-signal inactiv. 
//               Setting of parameter and switching of port line
//               are allways done by ISR. Therefor the pulse counter is 
//               set to one to start generator.
//
//               Notice: A channel must not switch on, 
//                       if channel already activ.
//               
//
// Parameters:   bChannel_p:       index of channel
//               bitRun:           TRUE  -> start channel
//                                 FALSE -> stop channel
//
// 
// Returns:      errorcode
//
//
// State:        tested (18.06.2004 A.S.)
//
//---------------------------------------------------------------------------

BYTE PUBLIC C14PtoEnable(BYTE bChannel_p, bit bitRun_p)
{
    switch (bChannel_p)
    {
        // -----------------------------------------------
        case PTO0: 
             if (bitRun_p)
             {
                // Setting of parameter and switching of port line
                // are allways done by ISR. Therefor the pulse counter is 
                // set to one to start generator.
                dwPTO0ImpulseCounter_l = 1;
                //---------------------------------------------------------
                if (wMode_l & BASE_CLOCK_MSEC)
                {  // base clock 1ms
                   wBase0ImpulseCounter_l = -1;;
                   C14IsrSetCC26 (TRUE, C14Pto0HandlerCC26);
                   CC2_CC26IC_IR  = 1;
                }
                else
                {
                   // base clock 800ns
                   C14IsrSetCC24 (TRUE, C14Pto0HandlerCC24);
                   CC2_CC24IC_IR  = 1;
                }
             }
             else
             {
                // stop alternative function of port line -> controlling 
                // port line by process image
                ALTSEL1P9 &= ~MASK_CHN_0;
                POUT_CHN0 = CHANNEL_INACTIV;
                // stop interrupts, CC16, CC24, CC26 stop
                C14IsrSetCC26 (FALSE, NULL);
                C14IsrSetCC24 (FALSE, NULL);
                // reset segment table
                wPto0ProfileWrIdx_l = 0;
                wPto0ProfileRdIdx_l = 0;
                // reset flags
                wError_l &= ~(OVERRUN_0 | ABORT_0 | DELTA_ERR_0);
             }
        break;
                   
        // -----------------------------------------------
        case PTO1: 
             if (bitRun_p)
             {
                // Setting of parameter and switching of port line
                // are allways done by ISR. Therefor the pulse counter is 
                // set to one to start generator.
                dwPTO1ImpulseCounter_l = 1;
                //---------------------------------------------------------
                if (wMode_l & (BASE_CLOCK_MSEC << 1))
                {  // base clock 1ms
                   wBase1ImpulseCounter_l = -1;;
                   C14IsrSetCC27 (TRUE, C14Pto1HandlerCC27);
                   CC2_CC27IC_IR  = 1;
                }
                else
                {
                   // base clock 800ns
                   C14IsrSetCC25 (TRUE, C14Pto1HandlerCC25);
                   CC2_CC25IC_IR  = 1;
                }
             }
             else
             {
                // stop alternative function of port line -> controlling 
                // port line by process image
                ALTSEL1P9 &= ~MASK_CHN_1;
                POUT_CHN1 = CHANNEL_INACTIV;
                // stop interrupts, CC16, CC24, CC26 stop
                C14IsrSetCC27 (FALSE, NULL);
                C14IsrSetCC25 (FALSE, NULL);
                // reset segment table
                wPto1ProfileWrIdx_l = 0;
                wPto1ProfileRdIdx_l = 0;
                // reset flags
                wError_l &= ~(OVERRUN_1 | ABORT_1 | DELTA_ERR_1);
             }
        break;

        // -----------------------------------------------
        default:
             return (C14_INVALID_CHANNEL);
        break;
                   
    }

    return (C14_SUCCESSFUL);
}

//---------------------------------------------------------------------------
//
// Function:     C14Pto0HandlerCC24
//
// Description:  The interrupt handler calculates for the PTO channel P0  
//               the new periode time and pulse width and controls the
//               number of transferred pulses. The periode time is changed  
//               by Delta. After a pulse sequence is completely transferred,  
//               the next segment is read and the transfer is re-starts with 
//               new parameters. If all segments is transferred, the PTO generator
//               is finished. Than controlling port line is done by process image. 
//
//               If the result of new periode time overflows the PTO generator stops
//               and the port line is controlled by process image.
// 
// Parameters:   none
//
// 
// Returns:      none
//
//
// State:        tested (18.06.2004 A.S.)
//
//---------------------------------------------------------------------------

static void C14Pto0HandlerCC24(void)
{
long     lPeriode;
WORD     wNextBuffRdIdx;
long     liDelta = (long)iPTO0Delta_l;

    dwPTO0ImpulseCounter_l--;
    if (!dwPTO0ImpulseCounter_l)
    {
       // one segment are already transferred 
       // re-start the next segment?
       if (wPto0ProfileWrIdx_l - wPto0ProfileRdIdx_l)
       {
          POUT_CHN0 = CHANNEL_INACTIV;
          ALTSEL1P9 |= MASK_CHN_0;
          liDelta                = 0; // For the first calculation must the Delta = 0.
          wPWM0Periode_l         = aPto0Profile_l[wPto0ProfileRdIdx_l].m_wPeriode;
          iPTO0Delta_l           = aPto0Profile_l[wPto0ProfileRdIdx_l].m_iDelta;
          dwPTO0ImpulseCounter_l = aPto0Profile_l[wPto0ProfileRdIdx_l].m_dwImpulse;

          wNextBuffRdIdx = wPto0ProfileRdIdx_l;
          wNextBuffRdIdx += 1;
          if (wNextBuffRdIdx >= tabentries(aPto0Profile_l))
          {
             wNextBuffRdIdx = 0;
          }

          wPto0ProfileRdIdx_l = wNextBuffRdIdx;
       }
       // there is no another segment --> stop PTO generator
       else
       {
          ALTSEL1P9 &= ~MASK_CHN_0;
          C14IsrSetCC24 (FALSE, NULL);
          POUT_CHN0 = CHANNEL_INACTIV;
          return;
       }
    }

    //-----------------------------------------------------------
    // calculate the new value of periode time 
    lPeriode = (long)((DWORD)wPWM0Periode_l) + liDelta;
    if ( (lPeriode >= 0x10000) || (lPeriode < 125))
    {
       // overflow --> PTO generator stops
       ALTSEL1P9 &= ~MASK_CHN_0;
       C14IsrSetCC24 (FALSE, NULL);
       POUT_CHN0 = CHANNEL_INACTIV;
       wError_l |= DELTA_ERR_0;
       return;
    }

    wPWM0Periode_l = (WORD)lPeriode;

    //-----------------------------------------------------------
    CC2_T78CON_T7R = 0;
    CC2_SEE |=  0x0101;  // Single-Periode
    POUT_CHN0 = CHANNEL_ACTIV;
    CC2_CC24 = CC2_T7 + wPWM0Periode_l;
    CC2_CC16 = CC2_T7 + (wPWM0Periode_l >> 1); // PTO: pulse time = 50% of periode time
    CC2_T78CON_T7R = 1;
}


//---------------------------------------------------------------------------
//
// Function:     C14Pto1HandlerCC25
//
// Description:  The interrupt handler calculates for the PTO channel P1  
//               the new periode time and pulse width and controls the
//               number of transferred pulses. The periode time is changed  
//               by Delta. After a pulse sequence is completely transferred,  
//               the next segment is read and the transfer is re-starts with 
//               new parameters. If all segments is transferred, the PTO generator
//               is finished. Than controlling port line is done by process image. 
//
//               If the result of new periode time overflows the PTO generator stops
//               and the port line is controlled by process image.
// 
// Parameters:   none
//
// 
// Returns:      none
//
//
// State:        tested (18.06.2004 A.S.)
//
//---------------------------------------------------------------------------

static void C14Pto1HandlerCC25(void)
{
long     lPeriode;
WORD     wNextBuffRdIdx;
long     liDelta = (long)iPTO1Delta_l;

    dwPTO1ImpulseCounter_l--;
    if (!dwPTO1ImpulseCounter_l)
    {
       // one segment are already transferred 
       // re-start the next segment?
       if (wPto1ProfileWrIdx_l - wPto1ProfileRdIdx_l)
       {
          POUT_CHN1 = CHANNEL_INACTIV;
          ALTSEL1P9 |= MASK_CHN_1;
  
          liDelta                = 0; // For the first calculation must the Delta = 0.
          wPWM1Periode_l         = aPto1Profile_l[wPto1ProfileRdIdx_l].m_wPeriode;
          iPTO1Delta_l           = aPto1Profile_l[wPto1ProfileRdIdx_l].m_iDelta;
          dwPTO1ImpulseCounter_l = aPto1Profile_l[wPto1ProfileRdIdx_l].m_dwImpulse;

          wNextBuffRdIdx = wPto1ProfileRdIdx_l;
          wNextBuffRdIdx += 1;
          if (wNextBuffRdIdx >= tabentries(aPto1Profile_l))
          {
             wNextBuffRdIdx = 0;
          }

          wPto1ProfileRdIdx_l = wNextBuffRdIdx;
       }
       // there is no another segment --> stop PTO generator
       else
       {
          ALTSEL1P9 &= ~MASK_CHN_1;
          C14IsrSetCC25 (FALSE, NULL);
          POUT_CHN1 = CHANNEL_INACTIV;
          return;
       }
    }

    //-----------------------------------------------------------
    // calculate the new value of periode time 
    lPeriode = (long)((DWORD)wPWM1Periode_l) + liDelta;
    if ( (lPeriode >= 0x10000) || (lPeriode < 125))
    {
       // overflow --> PTO generator stops
       ALTSEL1P9 &= ~MASK_CHN_1;
       C14IsrSetCC25 (FALSE, NULL);
       POUT_CHN1 = CHANNEL_INACTIV;
       wError_l |= DELTA_ERR_1;
       return;
    }

    wPWM1Periode_l = (WORD)lPeriode;

    //-----------------------------------------------------------
    CC2_T78CON_T7R = 0;
    CC2_SEE |=  0x0202;  // Single-Periode
    POUT_CHN1 = CHANNEL_ACTIV;
    CC2_CC25 = CC2_T7 + wPWM1Periode_l; 
    CC2_CC17 = CC2_T7 + (wPWM1Periode_l >> 1); // PTO: pulse time = 50% of periode time
    CC2_T78CON_T7R = 1;
}

//---------------------------------------------------------------------------
//
// Function:     C14Pto0HandlerCC26
//
// Description:  The interrupt handler calculates for the PTO channel P0  
//               the new periode time and pulse width and controls the
//               number of transferred pulses. The periode time is changed  
//               by Delta. After a pulse sequence is completely transferred,  
//               the next segment is read and the transfer is re-starts with 
//               new parameters. If all segments is transferred, the PTO generator
//               is finished. Than controlling port line is done by process image. 
//
//               If the result of new periode time overflows the PTO generator stops
//               and the port line is controlled by process image.
// 
// Parameters:   none
//
// 
// Returns:      none
//
//
// State:        tested (18.06.2004 A.S.)
//
//---------------------------------------------------------------------------

static void C14Pto0HandlerCC26(void)
{
long     lPeriode;
WORD     wNextBuffRdIdx;
long     liDelta = (long)iPTO0Delta_l;

    // re-start the next base clock periode of 1ms
    CC2_CC26 = CC2_T7 + PWM_TICKS_PER_MSEC;

    //-----------------------------------------------------------
    // start a new PTO pulse sequence?
    if (wBase0ImpulseCounter_l == -1)
    {
        dwPTO0ImpulseCounter_l--;
        if (!dwPTO0ImpulseCounter_l)
        {
           // one segment are already transferred 
           // re-start the next segment?
           if (wPto0ProfileWrIdx_l - wPto0ProfileRdIdx_l)
           {
              POUT_CHN0 = CHANNEL_INACTIV;
              ALTSEL1P9 |= MASK_CHN_0;
      
              liDelta                = 0; // For the first calculation must the Delta = 0.
              wPWM0Periode_l         = aPto0Profile_l[wPto0ProfileRdIdx_l].m_wPeriode;
              iPTO0Delta_l           = aPto0Profile_l[wPto0ProfileRdIdx_l].m_iDelta;
              dwPTO0ImpulseCounter_l = aPto0Profile_l[wPto0ProfileRdIdx_l].m_dwImpulse;
    
              wNextBuffRdIdx = wPto0ProfileRdIdx_l;
              wNextBuffRdIdx += 1;
              if (wNextBuffRdIdx >= tabentries(aPto0Profile_l))
              {
                 wNextBuffRdIdx = 0;
              }
    
              wPto0ProfileRdIdx_l = wNextBuffRdIdx;
           }
           // there is no another segment --> stop PTO generator
           else
           {
              ALTSEL1P9 &= ~MASK_CHN_0;
              C14IsrSetCC26 (FALSE, NULL);
              POUT_CHN0 = CHANNEL_INACTIV;
              return;
           }
        }
        //-----------------------------------------------------------
        // calculate the new value of periode time 
        lPeriode = (long)((DWORD)wPWM0Periode_l) + liDelta;
        if ( (lPeriode >= 0x10000) || (lPeriode < 2))
        {
           // overflow --> PTO generator stops
           ALTSEL1P9 &= ~MASK_CHN_0;
           C14IsrSetCC26 (FALSE, NULL);
           POUT_CHN0 = CHANNEL_INACTIV;
           wError_l |= DELTA_ERR_0;
           return;
        }
    
        wPWM0Periode_l = (WORD)lPeriode;
    }

    //----------------------------------------------------------------
    // counting pulses of base clock and controling port state
    wBase0ImpulseCounter_l++;
    // Number of base clocks to switch on PWM line reached?
    // If on-time 0%, then channel allways off.
    if (wBase0ImpulseCounter_l < (wPWM0Periode_l >> 1))
    {
       POUT_CHN0 = CHANNEL_ACTIV;
       return;
    }

    // periode time over?
    // If on-time 100%, then channel allways on.
    if (wBase0ImpulseCounter_l >= wPWM0Periode_l)
    {
       wBase0ImpulseCounter_l = -1;
       // re-trigger ISR to start the next periode
       CC2_CC26IC_IR = 1;
       return;
    }

    // puls time over --> channel switchs off
    POUT_CHN0 = CHANNEL_INACTIV;
}

//---------------------------------------------------------------------------
//
// Function:     C14Pto1HandlerCC27
//
// Description:  The interrupt handler calculates for the PTO channel P1  
//               the new periode time and pulse width and controls the
//               number of transferred pulses. The periode time is changed  
//               by Delta. After a pulse sequence is completely transferred,  
//               the next segment is read and the transfer is re-starts with 
//               new parameters. If all segments is transferred, the PTO generator
//               is finished. Than controlling port line is done by process image. 
//
//               If the result of new periode time overflows the PTO generator stops
//               and the port line is controlled by process image.
// 
// Parameters:   none
//
// 
// Returns:      none
//
//
// State:        tested (18.06.2004 A.S.)
//
//---------------------------------------------------------------------------

static void C14Pto1HandlerCC27(void)
{
long     lPeriode;
WORD     wNextBuffRdIdx;
long     liDelta = (long)iPTO1Delta_l;

    // re-start the next base clock periode of 1ms
    CC2_CC27 = CC2_T7 + PWM_TICKS_PER_MSEC;

    //-----------------------------------------------------------
    // start a new PTO pulse sequence?
    if (wBase1ImpulseCounter_l == -1)
    {
        dwPTO1ImpulseCounter_l--;
        if (!dwPTO1ImpulseCounter_l)
        {
           // one segment are already transferred 
           // re-start the next segment?
           if (wPto1ProfileWrIdx_l - wPto1ProfileRdIdx_l)
           {
              POUT_CHN1 = CHANNEL_INACTIV;
              ALTSEL1P9 |= MASK_CHN_1;
      
              liDelta                = 0; // For the first calculation must the Delta = 0.
              wPWM1Periode_l         = aPto1Profile_l[wPto1ProfileRdIdx_l].m_wPeriode;
              iPTO1Delta_l           = aPto1Profile_l[wPto1ProfileRdIdx_l].m_iDelta;
              dwPTO1ImpulseCounter_l = aPto1Profile_l[wPto1ProfileRdIdx_l].m_dwImpulse;

              wNextBuffRdIdx = wPto1ProfileRdIdx_l;
              wNextBuffRdIdx += 1;
              if (wNextBuffRdIdx >= tabentries(aPto1Profile_l))
              {
                 wNextBuffRdIdx = 0;
              }
    
              wPto1ProfileRdIdx_l = wNextBuffRdIdx;
           }
           // there is no another segment --> stop PTO generator
           else
           {
              ALTSEL1P9 &= ~MASK_CHN_1;
              C14IsrSetCC27 (FALSE, NULL);
              POUT_CHN1 = CHANNEL_INACTIV;
              return;
           }
        }
        //-----------------------------------------------------------
        // calculate the new value of periode time 
        lPeriode = (long)((DWORD)wPWM1Periode_l) + liDelta;
        if ( (lPeriode >= 0x10000) || (lPeriode < 2))
        {
           ALTSEL1P9 &= ~MASK_CHN_1;
           C14IsrSetCC27 (FALSE, NULL);
           POUT_CHN1 = CHANNEL_INACTIV;
           wError_l |= DELTA_ERR_1;
           return;
        }
    
        wPWM1Periode_l = (WORD)lPeriode;
    }

    //----------------------------------------------------------------
    // counting pulses of base clock and controling port state
    wBase1ImpulseCounter_l++;
    // Number of base clocks to switch on PWM line reached?
    // If on-time 0%, then channel allways off.
    if (wBase1ImpulseCounter_l < (wPWM1Periode_l >> 1))
    {
       // Kanal einschalten
       POUT_CHN1 = CHANNEL_ACTIV;
       return;
    }

    // periode time over?
    // If on-time 100%, then channel allways on.
    if (wBase1ImpulseCounter_l >= wPWM1Periode_l)
    {
       wBase1ImpulseCounter_l = -1;
       // re-trigger ISR to start the next periode
       CC2_CC27IC_IR = 1;
       return;
    }

    // puls time over --> channel switchs off
    POUT_CHN1 = CHANNEL_INACTIV;
}


//---------------------------------------------------------------------------
//
// Function:     C14IsrSetCC24
//
// Description:  The function enables or disables the interrupt CC24 
//               channel and install the handler. 
// 
// Parameters:   bState_p
//               TRUE  -> interrupt enable
//               FALSE -> interrupt disable
//
//               fpIsrHandler_p
// 
// Returns:      error code 
//
//
// State:        tested (18.06.2004 A.S.)
//
//---------------------------------------------------------------------------

static BYTE C14IsrSetCC24 (BYTE bState_p, void (*fpIsrHandler_p)(void))
{
    if (bState_p != FALSE)
    {
       if (fpIsrHandler_p == NULL)
          return (C14_INVALID_PARAMETER);

       fpC14HandlerCC24_l = fpIsrHandler_p;

       CC2_SEM    |= 0x0101;
       CC2_M4     |= 0x0005;
       CC2_M6     |= 0x0004;
       CC2_CC24IC |= 0x40 + INT_PRIORITY_PWM0;
    }
    else
    {
       CC2_CC24IC = 0;
       // CC16, CC24, CC26 stop
       CC2_M4  &= ~0x000F;
       CC2_M6  &= ~0x000F;
       CC2_SEM &= ~0x0101;
       CC2_SEE &= ~0x0101;
       fpC14HandlerCC24_l = NULL;
    }

    return (C14_SUCCESSFUL);
}


//---------------------------------------------------------------------------
//
// Function:     C14IsrSetCC25
//
// Description:  The function enables or disables the interrupt CC25 
//               channel and install the handler. 
// 
// Parameters:   bState_p
//               TRUE  -> interrupt enable
//               FALSE -> interrupt disable
//
//               fpIsrHandler_p
// 
// Returns:      error code 
//
//
// State:        tested (18.06.2004 A.S.)
//
//---------------------------------------------------------------------------

static BYTE C14IsrSetCC25 (BYTE bState_p, void (*fpIsrHandler_p)(void))
{
    if (bState_p != FALSE)
    {
       if (fpIsrHandler_p == NULL)
          return (C14_INVALID_PARAMETER);

       fpC14HandlerCC25_l = fpIsrHandler_p;

       CC2_SEM    |= 0x0202;
       CC2_M4     |= 0x0050;
       CC2_M6     |= 0x0040;
       CC2_CC25IC |= 0x40 + INT_PRIORITY_PWM1;
    }
    else
    {
       CC2_CC25IC = 0;
       CC2_M4  &= ~0x00F0;
       CC2_M6  &= ~0x00F0;
       CC2_SEM &= ~0x0202;
       CC2_SEE &= ~0x0202;
       fpC14HandlerCC25_l = NULL;
    }

    return (C14_SUCCESSFUL);
}


//---------------------------------------------------------------------------
//
// Function:     C14IsrSetCC26
//
// Description:  The function enables or disables the interrupt CC26
//               channel and install the handler. 
// 
// Parameters:   bState_p
//               TRUE  -> interrupt enable
//               FALSE -> interrupt disable
//
//               fpIsrHandler_p
// 
// Returns:      error code 
//
//
// State:        tested (18.06.2004 A.S.)
//
//---------------------------------------------------------------------------

static BYTE C14IsrSetCC26 (BYTE bState_p, void (*fpIsrHandler_p)(void))
{
    if (bState_p != FALSE)
    {
       if (fpIsrHandler_p == NULL)
          return (C14_INVALID_PARAMETER);

       fpC14HandlerCC26_l = fpIsrHandler_p;
       CC2_M6     |= 0x0400; // P0: Compare Mode 0, T7
       CC2_CC26IC |= 0x40 + INT_PRIORITY_PWM0;
    }
    else
    {
       CC2_CC26IC = 0;
       CC2_M6    &= ~0x0F00;
       fpC14HandlerCC26_l = NULL;
    }

    return (C14_SUCCESSFUL);
}

//---------------------------------------------------------------------------
//
// Function:     C14IsrSetCC27
//
// Description:  The function enables or disables the interrupt CC27 
//               channel and install the handler. 
// 
// Parameters:   bState_p
//               TRUE  -> interrupt enable
//               FALSE -> interrupt disable
//
//               fpIsrHandler_p
// 
// Returns:      error code 
//
//
// State:        tested (18.06.2004 A.S.)
//
//---------------------------------------------------------------------------

static BYTE C14IsrSetCC27 (BYTE bState_p, void (*fpIsrHandler_p)(void))
{
    if (bState_p != FALSE)
    {
       if (fpIsrHandler_p == NULL)
          return (C14_INVALID_PARAMETER);

       fpC14HandlerCC27_l = fpIsrHandler_p;
       CC2_M6     |= 0x4000; // P0: Compare Mode 0, T7
       CC2_CC27IC |= 0x40 + INT_PRIORITY_PWM1;
    }
    else
    {
       CC2_CC27IC = 0;
       CC2_M6  &= ~0xF000;
       fpC14HandlerCC27_l = NULL;
    }

    return (C14_SUCCESSFUL);
}


//---------------------------------------------------------------------------
//
// Function:     C14IsrCC24
//
// Description:  Interrupt service function (ISR) for CC24 channel
// 
// Parameters:   none
//
// 
// Returns:      none
//
//
// State:        tested (18.06.2004 A.S.)
//
//---------------------------------------------------------------------------

static void C14IsrCC24 (void) interrupt 56 using REG_BANK_CHN0
{
    if (fpC14HandlerCC24_l)
      fpC14HandlerCC24_l();
}

//---------------------------------------------------------------------------
//
// Function:     C14IsrCC25
//
// Description:  Interrupt service function (ISR) for CC25 channel
// 
// Parameters:   none
//
// 
// Returns:      none
//
//
// State:        tested (18.06.2004 A.S.)
//
//---------------------------------------------------------------------------

static void C14IsrCC25 (void) interrupt 57 using REG_BANK_CHN1
{
    if (fpC14HandlerCC25_l)
       fpC14HandlerCC25_l();
}

//---------------------------------------------------------------------------
//
// Function:     C14IsrCC26
//
// Description:  Interrupt service function (ISR) for CC26 channel
// 
// Parameters:   none
//
// 
// Returns:      none
//
//
// State:        tested (18.06.2004 A.S.)
//
//---------------------------------------------------------------------------

static void C14IsrCC26 (void) interrupt 58 using REG_BANK_CHN0
{
    if (fpC14HandlerCC26_l)
       fpC14HandlerCC26_l();
}

//---------------------------------------------------------------------------
//
// Function:     C14IsrCC27
//
// Description:  Interrupt service function (ISR) for CC27 channel
// 
// Parameters:   none
//
// 
// Returns:      none
//
//
// State:        tested (18.06.2004 A.S.)
//
//---------------------------------------------------------------------------

static void C14IsrCC27 (void) interrupt 59 using REG_BANK_CHN1
{
    if (fpC14HandlerCC27_l)
       fpC14HandlerCC27_l();
}

