/****************************************************************************

  (c) SYSTEC electronic GmbH, D-07973 Greiz, August-Bebel-Str. 29
      www.systec-electronic.de

  Testprogramm fuer Rtos16x

  11.08.2003 -rs:   Initial version

****************************************************************************/

#include <stdio.h>
#include <XC16x.h>

// Relativ zum globalen Pfad ....
#include "generic/global.h"

// Relativ zum Projekt-Pfad
#include "../include/rtos.h"

BYTE bFlag = 1;

void TestHandler(void)
{
}

void TestHandlerWdt(void)
{
    // Watchdog timer abgelaufen, sollte nicht passieren!
    // Durch die Applikation knnte hier ein Flag gesetzt 
    // werden, dass im Hauptprogramm geprft und diesen Zustand
    // signalisiert (Fehlermeldung).
}

void TestHandlerCOP(void)
{
    RtosDelayTask(TASK_TMR_COM,   10000);
}


void TestRtosTask(BYTE bTaskNo_p)
{
    RtosCreateTask(bTaskNo_p, NULL);
    RtosDelayTask(bTaskNo_p, 1000L);
    RtosDeleteTask(bTaskNo_p);

    RtosCreateTask(bTaskNo_p, TestHandler);
    RtosSendSignal(bTaskNo_p);

    RtosDisableTask(bTaskNo_p);
    RtosSendSignal(bTaskNo_p);
    RtosClearSignal(bTaskNo_p);
    RtosEnableTask(bTaskNo_p);
    RtosDeleteTask(bTaskNo_p);
}

void main()
{

    // Initialisierung interene Schnittstelle S0 mit 9600Bd
    // (Initialisierung nur dann, wenn Schnittstelle nicht schon bereits
    // initalisiert ist, z.B. durch Monitor)
    if (ASC0_CON == 0x00)
    {
        P3    |= 0x0400;      // P3.10 setzen (TxD)
        DP3   |= 0x0400;      // P3.10 auf Ausgang setzen (TxD)
        DP3   &= 0xF7FF;      // P3.11 auf Eingang setzen (RxD)
        ASC0_TIC  = 0x80;     // TI-Flag setzen
        ASC0_RIC  = 0x00;     // RI-Flag loeschen
        ASC0_BG   = 0x40;     // BR=57600: 0x0A, BR=9600: 0x40
        ASC0_CON  = 0x8011;   // Modus setzen
        ALTSEL0P3|= 0x0C00;   // Configure port pins for serial interface 0 
    }


    printf ("\nRtosInit...");
    RtosInit();

    PSW_IEN = 1;

    TestRtosTask(TASK_ADC_ERROR);
    TestRtosTask(TASK_TMR_COM);
    TestRtosTask(TASK_TMR_WDT);
    TestRtosTask(TASK_PLL_ERROR);

    printf ("\n");

    RtosInitWatchdog(TestHandlerWdt);

    RtosCreateTask(TASK_ADC_ERROR, TestHandler);
    RtosCreateTask(TASK_TMR_COM,   TestHandlerCOP);
    RtosCreateTask(TASK_PLL_ERROR, TestHandler);

    RtosDelayTask(TASK_TMR_COM,   200);


    while(bFlag)
       RtosResetWatchdog();

    RtosShutDownWatchdog();
    RtosShutDown(RTOS_SHUTDOWN_USER_LEVEL);

    while(1);
}

// EOF



