/****************************************************************************

  (c) SYSTEC electronic GmbH, D-07973 Greiz, August-Bebel-Str. 29
      www.systec-electronic.de

  Project:      Serial driver

  Description:  Serial driver interface
                The interface is independent of target or hardware.

  -------------------------------------------------------------------------

                $RCSfile: SioSys.c,v $
                
                $Author: A.Suess $
                
                $Revision: 1.1.1.1 $  $Date: 2004/09/16 11:56:05 $
                
                $State: Exp $
                
                Build Environment:
                KEIL uVision 2

  -------------------------------------------------------------------------

  Revision History:

  2003/07/07 -avc:   Start der Implementierung
  2003/10/22 -as:    Kommentare ergnzt
  2004/02/17 -ct:    Angepasst fr interne ASC0 vom XC161

****************************************************************************/

#include "generic/global.h"
#include "generic/gmacros.h"
#include "../include/siodrv.h"




/***************************************************************************/
/*                                                                         */
/*                                                                         */
/*          G L O B A L   D E F I N I T I O N S                            */
/*                                                                         */
/*                                                                         */
/***************************************************************************/

//---------------------------------------------------------------------------
// const defines
//---------------------------------------------------------------------------

// number of serial instances
#define INSTANCE_COUNT  3



//---------------------------------------------------------------------------
// local function prototypes
//---------------------------------------------------------------------------

//--------------------------------------------------------------------------
// function prototypes for instance asc0
BOOL ASC0_SioInit           (tSioCfgParam FAR* SioCfgParam);
BOOL ASC0_SioClose          (void);
BOOL ASC0_SioTxFifoPutByte  (char bValue);
BOOL ASC0_SioRxFifoGetByte  (char FAR* pbValue);
BOOL ASC0_SioTxFifoDataSize (DWORD FAR* pdwSize_p);
BOOL ASC0_SioRxFifoDataSize (DWORD FAR* pdwSize_p);
BOOL ASC0_SioGetState       (tSioState FAR* pSioState_p);
BOOL ASC0_SioSetState       (tSioState FAR* pSioState_p);

//--------------------------------------------------------------------------
// function prototypes for instance asc1
BOOL ASC1_SioInit           (tSioCfgParam FAR* SioCfgParam);
BOOL ASC1_SioClose          (void);
BOOL ASC1_SioTxFifoPutByte  (char bValue);
BOOL ASC1_SioRxFifoGetByte  (char FAR* pbValue);
BOOL ASC1_SioTxFifoDataSize (DWORD FAR* pdwSize_p);
BOOL ASC1_SioRxFifoDataSize (DWORD FAR* pdwSize_p);
BOOL ASC1_SioGetState       (tSioState FAR* pSioState_p);
BOOL ASC1_SioSetState       (tSioState FAR* pSioState_p);

//--------------------------------------------------------------------------
// function prototypes for instance UART
BOOL S1_SioInit             (tSioCfgParam FAR* SioCfgParam);
BOOL S1_SioClose            (void);
BOOL S1_SioTxFifoPutByte    (char bValue);
BOOL S1_SioRxFifoGetByte    (char FAR* pbValue);
BOOL S1_SioTxFifoDataSize   (DWORD FAR* pdwSize_p);
BOOL S1_SioRxFifoDataSize   (DWORD FAR* pdwSize_p);
BOOL S1_SioGetState         (tSioState FAR* pSioState_p);
BOOL S1_SioSetState         (tSioState FAR* pSioState_p);



//---------------------------------------------------------------------------
// modul globale vars
//---------------------------------------------------------------------------

static BYTE FAR  abSioState_l[INSTANCE_COUNT];


// Instance table of interface
CONST tSerialInst aSioInstTab_l[] =
{
    //-----------------------------------------------------
    // Parameter for instance 0 = ASC0
    #ifdef _ASC0_
    {
        (BYTE FAR*) &abSioState_l[0],
        ASC0_SioGetState,
        ASC0_SioSetState,
        ASC0_SioRxFifoGetByte,
        ASC0_SioTxFifoPutByte,
        ASC0_SioTxFifoDataSize,
        ASC0_SioRxFifoDataSize,
        ASC0_SioInit,
        ASC0_SioClose
    },
    #else
    {
        NULL,
        NULL,
        NULL,
        NULL,
        NULL,
        NULL,
        NULL,
        NULL,
        NULL
    },
    #endif


    //-----------------------------------------------------
    // Parameter for instance 1 = ASC1
    #ifdef _ASC1_
    {
        (BYTE FAR*) &abSioState_l[1],
        ASC1_SioGetState,
        ASC1_SioSetState,
        ASC1_SioRxFifoGetByte,
        ASC1_SioTxFifoPutByte,
        ASC1_SioTxFifoDataSize,
        ASC1_SioRxFifoDataSize,
        ASC1_SioInit,
        ASC1_SioClose
    },
    #else
    {
        NULL,
        NULL,
        NULL,
        NULL,
        NULL,
        NULL,
        NULL,
        NULL,
        NULL
    },
    #endif

    //-----------------------------------------------------
    // Parameter for instance 2 = UART
    #ifdef _UART_
    {
        (BYTE FAR*) &abSioState_l[2],
        S1_SioGetState,
        S1_SioSetState,
        S1_SioRxFifoGetByte,
        S1_SioTxFifoPutByte,
        S1_SioTxFifoDataSize,
        S1_SioRxFifoDataSize,
        S1_SioInit,
        S1_SioClose
    }
    #else
    {
        NULL,
        NULL,
        NULL,
        NULL,
        NULL,
        NULL,
        NULL,
        NULL,
        NULL
    }
    #endif
};





/***************************************************************************/
/*                                                                         */
/*                                                                         */
/*          C L A S S  <SerialInstance>                                    */
/*                                                                         */
/*                                                                         */
/***************************************************************************/
//
// Description:
// Die Klasse stellt eine Funktion zur Verfgung, um instanzabhngig
// auf die Funktionen und Daten der jeweiligen Treiber-Instanz zuzugreifen.
// Fr jede Instanz mu es einen Eintrag in der Instanztabelle aSioInstTab_l
// Die Instanztabelle enthlt die Funktionspointer auf die Interface-
// funktionen der jeweiligen Instanz. 
//
/***************************************************************************/

//=========================================================================//
//                                                                         //
//          P R I V A T E   D E F I N I T I O N S                          //
//                                                                         //
//=========================================================================//

//---------------------------------------------------------------------------
// const defines
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
// local types
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
// lokale vars
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
// local function prototypes
//---------------------------------------------------------------------------





//=========================================================================//
//                                                                         //
//          P U B L I C   F U N C T I O N S                                //
//                                                                         //
//=========================================================================//

//---------------------------------------------------------------------------
//
// Function:     SioGetInstData
//
// Description:  The function initialzes the instance data structures.
//
//
// Parameters:   none
//
// 
// Returns:      none
//
//
// State:        
//
//---------------------------------------------------------------------------

void  SioInitInstData (void)
{

WORD  nIdx;


    for (nIdx=0; nIdx < tabentries(abSioState_l); nIdx++)
    {
        abSioState_l[nIdx] = SIODRV_INACTIVE;
    }

}



//---------------------------------------------------------------------------
//
// Function:     SioGetInstData
//
// Description:  The function returns the pointer of an instance.
//
//
// Parameters:   bInstNum_P - number of related instance (0... )
//
// 
// Returns:      SUCCESS: address of instance table entry
//               ERROR:   NULL
//
// State:        
//
//---------------------------------------------------------------------------

tSerialInst FAR*  SioGetInstData (BYTE bInstNum_p)
{

    if (bInstNum_p < tabentries(aSioInstTab_l))
    {
        return ( (tSerialInst FAR*)&aSioInstTab_l[bInstNum_p] );
    }

    return (NULL);

}



