/****************************************************************************

  (c) SYSTEC electronic GmbH, D-07973 Greiz, August-Bebel-Str. 29
      www.systec-electronic.de

  Project:      PLCmodule C14

  Description:  serial driver for XC16x intern serial interface ASC

  -------------------------------------------------------------------------

                $RCSfile: XC_Asc1.c,v $
                
                $Author: A.Suess $

                $Revision: 1.1.1.1 $  $Date: 2004/09/16 11:56:05 $
                
                $State: Exp $
                
                Build Environment:
                KEIL uVision 2

  -------------------------------------------------------------------------

  Revision History:
   
  2004/02/16 -ct:    Start der Implemetierung mit bernahme des Source aus
                     "SioDrv_XC161_ASC0.c"

  2004/06/18 -as:    Definition von Makros fr die Auswahl der Schnittstelle 
                     ASC0 und ASC1

****************************************************************************/

#include <xc161.h>
#include <intrins.h>

#include "generic/global.h"
#include "../include/SioDrv.h"

// Auswahl interface
//#define ASC0
#define ASC1

/***************************************************************************/
/*                                                                         */
/*                                                                         */
/*          G L O B A L   D E F I N I T I O N S                            */
/*                                                                         */
/*                                                                         */
/***************************************************************************/

//---------------------------------------------------------------------------
// Definition der instanziierbaren ffentlichen Funktionsnamen
#ifdef ASC1
    #define SioInit                 ASC1_SioInit
    #define SioClose                ASC1_SioClose
    #define SioTxFifoPutByte        ASC1_SioTxFifoPutByte
    #define SioRxFifoGetByte        ASC1_SioRxFifoGetByte
    #define SioTxFifoDataSize       ASC1_SioTxFifoDataSize
    #define SioRxFifoDataSize       ASC1_SioRxFifoDataSize
    #define SioGetState             ASC1_SioGetState
    #define SioSetState             ASC1_SioSetState
    #define SioHwTxIsr              ASC1_TxISR
    #define SioHwRxIsr              ASC1_RxISR
#else
    #define SioInit                 ASC0_SioInit
    #define SioClose                ASC0_SioClose
    #define SioTxFifoPutByte        ASC0_SioTxFifoPutByte
    #define SioRxFifoGetByte        ASC0_SioRxFifoGetByte
    #define SioTxFifoDataSize       ASC0_SioTxFifoDataSize
    #define SioRxFifoDataSize       ASC0_SioRxFifoDataSize
    #define SioGetState             ASC0_SioGetState
    #define SioSetState             ASC0_SioSetState
    #define SioHwTxIsr              ASC0_TxISR
    #define SioHwRxIsr              ASC0_RxISR
#endif
//---------------------------------------------------------------------------
// const defines
//---------------------------------------------------------------------------
// Einstellung Gre der Ringpuffer
#define SERIAL_RX_FIFO_SIZE        254          // Size of RX/TX-FIFO in byte
#define SERIAL_TX_FIFO_SIZE        254          // Size of RX/TX-FIFO in byte

#if (SERIAL_RX_FIFO_SIZE < 49)
    #error 'Empfangspuffer zu klein'
#endif

// Reserved space in RX-FIFO for handshake buffering in byte (RTS is set to LOW if 
// this limit is reached) (default: 48)
#define SER_RXFIFO_STOP_LEVEL      SERIAL_RX_FIFO_SIZE-48

//---------------------------------------------------------------------------
// Einstellung des untersttzten Protokolle und Mode

// Control lines for hardware handshake supported: If not set to TRUE, then 
// XON-XOFF- or no protocol can be used.
//#define HW_HANDSHAKE_SUPPORT       TRUE      
#define HW_HANDSHAKE_SUPPORT       FALSE

// If USE_BLOCKED_MODE == TRUE, the function TxFifoPutByte is waiting
// until transmission is done.
//#define USE_BLOCKED_MODE           TRUE
#define USE_BLOCKED_MODE           FALSE

// If SIO_ECHO_SUPPORTED == TRUE, the upper layer functions can 
// controll sending of character echo.
#define SIO_ECHO_SUPPORTED         TRUE
//#define SIO_ECHO_SUPPORTED         FALSE

//---------------------------------------------------------------------------
// Konfiguration der Hardware
#define CPU_CLOCK                  40000000            // CPU cloch in Hz

// define the register, port and bitmask for TX and RX line
#ifdef ASC1
    // sfr definitions for ASC1
    #define ASCx_FDV        ASC1_FDV
    #define ASCx_BG         ASC1_BG
    #define ASCx_CON        ASC1_CON
    #define ASCx_TIC        ASC1_TIC
    #define ASCx_TIC_IE     ASC1_TIC_IE
    #define ASCx_TIC_IR     ASC1_TIC_IR
    #define ASCx_RIC        ASC1_RIC
    #define ASCx_RIC_IE     ASC1_RIC_IE
    #define ASCx_EIC        ASC1_EIC
    #define ASCx_TBUF       ASC1_TBUF
    #define ASCx_RBUF       ASC1_RBUF
    // interrupt definitions
    #define ASCx_TINT       72
    #define ASCx_RINT       73
    #define ASCx_EINT       74
    #define ASCx_TXINT_GPX  0
    #define ASCx_TXINT_ILVL 3
    #define ASCx_TXINT_GLVL 1
    #define ASCx_RXINT_GPX  0
    #define ASCx_RXINT_ILVL 4
    #define ASCx_RXINT_GLVL 1
    // port definitions
    #define COM_PORT        P3
    #define COM_PORT_MASK   0x0001 // define port 3.0 as TX-line for serial interface
    #define COM_DPORT       DP3
#else
    // sfr definitions for ASC0
    #define ASCx_FDV        ASC0_FDV
    #define ASCx_BG         ASC0_BG
    #define ASCx_CON        ASC0_CON
    #define ASCx_TIC        ASC0_TIC
    #define ASCx_TIC_IE     ASC0_TIC_IE
    #define ASCx_TIC_IR     ASC0_TIC_IR
    #define ASCx_RIC        ASC0_RIC
    #define ASCx_RIC_IE     ASC0_RIC_IE
    #define ASCx_EIC        ASC0_EIC
    #define ASCx_TBUF       ASC0_TBUF
    #define ASCx_RBUF       ASC0_RBUF
    // interrupt definitions
    #define ASCx_TINT       42
    #define ASCx_RINT       43
    #define ASCx_EINT       44
    #define ASCx_TXINT_GPX  0
    #define ASCx_TXINT_ILVL 3
    #define ASCx_TXINT_GLVL 0
    #define ASCx_RXINT_GPX  0
    #define ASCx_RXINT_ILVL 4
    #define ASCx_RXINT_GLVL 0
    // port definitions
    #define COM_PORT        P3
    #define COM_PORT_MASK   0x0400 // define port 3.10 as TX-line for serial interface
    #define COM_DPORT       DP3
#endif


//---------------------------------------------------------------------------
#if(HW_HANDSHAKE_SUPPORT != FALSE)
    
    //define the control lines which are supported by hardware
    #define RTS_SIGNAL_SUPPORT      TRUE
    #define DTR_SIGNAL_SUPPORT      FALSE
    #define CTS_SIGNAL_SUPPORT      TRUE
    #define DSR_SIGNAL_SUPPORT      FALSE
    #define DCD_SIGNAL_SUPPORT      FALSE
    #define RI_SIGNAL_SUPPORT       FALSE

    //define the port and the direction port to wich the 
    //handshake lines are connected 
    #if(RTS_SIGNAL_SUPPORT != FALSE)
        #define RTS_BASE      P3
        #define RTS_BITMASK   0x0040               // bit mask to set the direction for RTS port pin for output
        #define RTS_DPR       DP3 |= RTS_BITMASK;  // configure RTS for output
        sbit    S1_RTS =      RTS_BASE^6;          // define portpin for RTS signal
        #define RTS_STATE     ( ~S1_RTS )          // macro encapsulation, used to interface RTS signal,returns TRUE if signal is HIGH
        #define CLR_RTS       (S1_RTS=1)
        #define SET_RTS       (S1_RTS=0)
    #else
        #define RTS_STATE     SIGNAL_NOT_SUPPORTED // RTS signal not supported
        #define CLR_RTS
        #define SET_RTS
        #define RTS_DPR
    #endif

    #if(DTR_SIGNAL_SUPPORT != FALSE)
        #define DTR_BASE     P3
        #define DTR_BITMASK  0x0008               // bit mask to set the direction for DTR port pin
        #define DTR_DPR      DP3 |= DTR_BITMASK   // configure DCD for output
        sbit    S1_DTR =     DTR_BASE^3;          // define portpin for DTR signal
        #define DTR_STATE   ( ~S1_DTR )           // macro encapsulation, used to interface DTR signal, returns TRUE if signal is HIGH
        #define CLR_DTR     (S1_DTR=1)
        #define SET_DTR     (S1_DTR=0)
    #else
        #define DTR_STATE   SIGNAL_NOT_SUPPORTED  // DTR signal not supported
        #define CLR_DTR
        #define SET_DTR
        #define DTR_DPR
    #endif

    #if(CTS_SIGNAL_SUPPORT != FALSE)
        #define CTS_BASE    P3
        #define CTS_BITMASK 0x0020               //bit mask to set the direction for CTS port pin
        #define CTS_DPR     DP3 &= ~CTS_BITMASK  // configure CTS for input
        sbit    S1_CTS =    CTS_BASE^5;          // define portpin for CTS signal
        #define CTS_STATE   ( ~S1_CTS )          // Macro encapsulation, used to interface CTS signal, returns TRUE if signal is HIGH
        #define CLR_CTS
        #define SET_CTS
    #else
        #define CTS_STATE   SIGNAL_NOT_SUPPORTED // CTS signal not supported
        #define CTS_DPR
        #define CLR_CTS
        #define SET_CTS
    #endif
    
    #if(DSR_SIGNAL_SUPPORT != FALSE)
        #define DSR_BASE     P3    
        #define DSR_BITMASK  0x0020                // bit mask to set the direction for DSR port pin
        #define DSR_DPR      DP3 &= ~DSR_BITMASK   // configure DSR for input
        sbit    S1_DSR =     DSR_BASE^5;           // define portpin for DSR signal
        #define DSR_STATE    ( ~S1_DSR )           // macro encapsulation, used to interface DSR signal, returns TRUE if signal is HIGH
    #else
        #define DSR_STATE    SIGNAL_NOT_SUPPORTED  // DSR signal not supported
        #define DSR_DPR
    #endif

    #if(DCD_SIGNAL_SUPPORT != FALSE)
        #define DCD_BASE     P3    
        #define DCD_BITMASK  0x0010                //bit mask to set the direction for DCD port pin
        #define DCD_DPR      DP3 &= ~DCD_BITMASK   // configure DCD for input
        sbit    S1_DCD =     DCD_BASE^4;           // Define portpin for DCD signal
        #define DCD_STATE    ( ~S1_DCD )           // Macro encapsulation, used to interface DCD signal, returns TRUE if signal is HIGH
    #else
        #define DCD_STATE    SIGNAL_NOT_SUPPORTED  // DCD signal not supported
        #define DCD_DPR
    #endif

    #if(RI_SIGNAL_SUPPORT != FALSE)
        #define RI_BASE     P3
        #define RI_BITMASK  0x0080                 // bit mask to set the direction for RI port pin
        #define RI_DPR      DP3 &= ~RI_BITMASK     // configure CTS for input
        sbit    S1_RI =     RI_BASE^7;             // Define portpin for RI signal
        #define RI_STATE    ( ~S1_RI  )            // Macro encapsulation, used to interface RI signal, returns TRUE if signal is HIGH
    #else
       #define RI_STATE     SIGNAL_NOT_SUPPORTED   // RI signal not supported
       #define RI_DPR
    #endif

    // RTS wird hier erst mal gelscht. Nach erfolgreichem Abschlu der Initialisierung
    // wird dann das Signal gesetzt. Dadurch soll verhindert werden, das der Sender mit
    // der bertragung beginnt, noch bevor der Empfnger bereit ist. 
    #define RTSCTS_INIT() { CLR_RTS ; RTS_DPR ; CLR_DTR ; DTR_DPR ; CTS_DPR ; DSR_DPR ; DCD_DPR ; RI_DPR; }


#else // #if(HW_HANDSHAKE_SUPPORT)
    // Hardware-Handshake wird nicht untersttzt. Um XON-XOFF-Protokoll
    // zu untersttzen, wird das Makro fr CTS verwendet, um 
    // Softwareflags zu steuern.
    // CTS steuert den Sendvorgang. Wurde XOFF empfangen, so geht CTS nach FALSE.
    // Wurde XON empfangen, so geht CTS auf TRUE.
    // fXOFF steuert das Empfangen von Zeichen. Ist der Puffer voll, so wird
    // fXOFF gesetzt und XOFF gesendet. Ist der Puffer leer, so wird XON gesendet.
    static  BOOL fCTS = TRUE;

    #define RTS_STATE   TRUE
    #define CTS_STATE   fCTS
    #define DTR_STATE   SIGNAL_NOT_SUPPORTED
    #define DSR_STATE   SIGNAL_NOT_SUPPORTED
    #define DCD_STATE   SIGNAL_NOT_SUPPORTED
    #define RI_STATE    SIGNAL_NOT_SUPPORTED
    #define CLR_RTS     
    #define SET_RTS     
    #define CLR_CTS     (fCTS = FALSE)
    #define SET_CTS     (fCTS = TRUE)
    #define CLR_DTR
    #define SET_DTR
    #define RTS_SIGNAL_SUPPORT   FALSE
    #define DTR_SIGNAL_SUPPORT   FALSE
    #define RTSCTS_INIT()        { fXOFF_l=FALSE; fXON_l=FALSE; SET_CTS ;}

#endif // #if(HW_HANDSHAKE_SUPPORT)

#define XON  0x11
#define XOFF 0x13

//---------------------------------------------------------------------------
// local types
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
// modul globale vars
//---------------------------------------------------------------------------
static BYTE bUseProtocol_l;
static BOOL fXOFF_l;
static BOOL fXON_l;

//---------------------------------------------------------------------------
// local function prototypes
//---------------------------------------------------------------------------

BOOL SioTxFifoDataSize  (DWORD FAR* pdwSize_p);
BOOL SioRxFifoDataSize  (DWORD FAR* pdwSize_p);

/***************************************************************************/
/*                                                                         */
/*                                                                         */
/*          C L A S S  <Serial Driver generic part>                        */
/*                                                                         */
/*                                                                         */
/***************************************************************************/
//
// Description:
// Die Klasse stellt hardwareunabhngig Funktionen des seriellen Treibers 
// zur Verfgung. 
//
/***************************************************************************/


//=========================================================================//
//                                                                         //
//          P R I V A T E   D E F I N I T I O N S                          //
//                                                                         //
//=========================================================================//

//---------------------------------------------------------------------------
// const defines
//---------------------------------------------------------------------------

// This macro sets a status bit (x) on global status variable.
#define SETSTAT()      SioHwSignalTxIsr()
// This macro clears a status bit (x) on global status variable.
#define CLRSTAT()      SioHwDisableTxIsr()

//---------------------------------------------------------------------------
// local types
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
// modul globale vars
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
// local function prototypes
//---------------------------------------------------------------------------
static BOOL SioHwInit         (tSioCfgParam FAR* SioCfgParam);
static BOOL SioHwClose        (void);
static void SioHwDisableTxIsr (void);
static BYTE SioHwGetErrorState(void);
static void SioHwSignalTxIsr  (void);
static void SioHwPutChar      (char Char);
static char SioHwGetChar      (void);

static void SioTriggerTx      (void);
static void SioTriggerRx      (void);

static void SioRBufInit       (void);
static void SioRBufRxClear    (void); 
static void SioRBufTxClear    (void);
static BOOL SioRBufRxRead     (BYTE FAR* pbData_p);
static BOOL SioRBufTxRead     (BYTE FAR* pbData_p);
static BOOL SioRBufRxWrite    (BYTE bData_p);
static BOOL SioRBufTxWrite    (BYTE bData_p);
static WORD SioRBufRxDataSize (void);
static WORD SioRBufTxDataSize (void);

//=========================================================================//
//                                                                         //
//          P U B L I C   F U N C T I O N S                                //
//                                                                         //
//=========================================================================//

//---------------------------------------------------------------------------
//
// Function:     SioInit
//
// Description:  This function is called by application to initialise 
//               serial interface.
//
//
// Parameters:   configuration parameter
//
// 
// Returns:      error code
//
//
// State:        o.k. 2004/02/16 -ct
//
//---------------------------------------------------------------------------

BOOL PUBLIC SioInit(tSioCfgParam FAR* pSioCfgParam_p)
{
BOOL fRetcode = TRUE;

    bUseProtocol_l = 0;
    fXOFF_l        = FALSE;
    fXON_l         = FALSE;

    // Sende- und Empfangspuffer init.
    SioRBufInit();

    // Steuerleitungen init., Anfangswerte setzen
    // RTS = 0
    // DTR = 0
    RTSCTS_INIT();

    // per Default wird Echo nicht untersttzt
    pSioCfgParam_p->m_bProtocol &= ~SIODRV_ECHO_SUPPORTED;

    // initialise serial interface
    fRetcode = SioHwInit(pSioCfgParam_p);

    // Initialisieren erfolgreich ?
    if (fRetcode)
    {
       pSioCfgParam_p->m_bErrCode = kSioDrvInitOk;

       #if(SIO_ECHO_SUPPORTED != FALSE)
       {
          // Der Treiber hinterlegt in m_bProtocol, ob der
          // Echo-Support untersttzt wird.
          pSioCfgParam_p->m_bProtocol |= SIODRV_ECHO_SUPPORTED;
       }
       #endif

       // TX-Task sperren
       CLRSTAT();

       // Initial. des Treibers erfolgreich 
       // abgeschlossen, RTS = 1 setzen.
       SET_RTS;
       fXON_l = TRUE;
       // Die DTR-Leitung bleibt hier auf 0, DTR wird
       // aus der Applikation gesetzt. 
    }    

    return (fRetcode);
}


//---------------------------------------------------------------------------
//
// Function:     SioClose
//
// Description:  This function is called by application to deinitialise 
//               serial interface.
//
//
// Parameters:   none
//
// 
// Returns:      true
//
//
// State:        tested (18.06.2004, A.S.)
//
//---------------------------------------------------------------------------

BOOL PUBLIC SioClose(void)
{
    CLR_RTS;
    SioHwClose();
    return TRUE;
}


//---------------------------------------------------------------------------
//
// Function:     SioGetState
//
// Description:  The function gets the state of modem control lines.
//
//
// Parameters:   pSioState_p    -pointer to struc tSioState
// 
//
// Returns:      BOOL           -TRUE/FALSE
//
//
// State:        tested (18.06.2004, A.S.)
//
//---------------------------------------------------------------------------

BOOL PUBLIC SioGetState(tSioState FAR* pSioState_p)
{   
DWORD dwSize;

    pSioState_p->m_bErrCode = kSioDrvInitOk;

    //get number of characters in the Rx FIFO
    if(SioRxFifoDataSize(&dwSize))
    {
        pSioState_p->m_dwCbRxQ = dwSize;
    }// End if
    else
    {
        pSioState_p->m_dwCbRxQ = 0L;    
    }//End else

    //get number of characters in the Tx FIFO
    if(SioTxFifoDataSize(&dwSize))
    {
        pSioState_p->m_dwCbTxQ = dwSize;
    }//End if
    else
    {
        pSioState_p->m_dwCbTxQ = 0L;    
    }//End else

    pSioState_p->m_dwSoRxQ = SERIAL_RX_FIFO_SIZE;// size of Tx queue
    pSioState_p->m_dwSoTxQ = SERIAL_TX_FIFO_SIZE;// size of Rx queue

    // Hardware handshake and modem control lines
    pSioState_p->m_bCTS = CTS_STATE;
    pSioState_p->m_bDSR = DSR_STATE;
    pSioState_p->m_bDCD = DCD_STATE;
    pSioState_p->m_bRI  = RI_STATE;
    pSioState_p->m_bRTS = RTS_STATE;
    pSioState_p->m_bDTR = DTR_STATE;

    //status register
    pSioState_p->m_wSioStat = (WORD)SioHwGetErrorState();

    SioTriggerTx();
    SioTriggerRx();

    return (TRUE);   
}

//---------------------------------------------------------------------------
//
// Function:     SioSetState
//
// Description:  The function sets the state of modem control lines.
//
//               The modem control lines are only effected, if 
//               the the hardware protocol was selected. 
//
//               If software protocol XON/XOFF or no protocoll is used, 
//               the state of RTS and CTS signal can not be changed.
//
// Parameters:   pSioState_p    -Pointer auf Statusstructur
// 
//
// Returns:      TRUE:  sucessfull 
//               FALSE: funktion finishs with error
//                      error code --> pSioState_p->m_bErrCode
//
//
// State:        tested (18.06.2004, A.S.)
//
//---------------------------------------------------------------------------

BOOL PUBLIC SioSetState(tSioState FAR* pSioState_p)
{
BYTE bRetcode = TRUE;

    pSioState_p->m_bErrCode = kSioDrvStatOk;

    // Lschen Empfangspuffer, Sendpuffer ?
    switch (pSioState_p->m_bClr)
    {
        // Empfangspuffer lschen
        case CLR_RX_BUFFER:
            SioRBufRxClear();
        break;

        // Sendepuffer lschen
        case CLR_TX_BUFFER:
            SioRBufTxClear();
        break;

        // Empfangspuffer und Sendepuffer lschen
        case CLR_RX_TX_BUFFER:
            SioRBufRxClear();
            SioRBufTxClear();
        break;

        default:
        break;  
    }

    //---------------------------------------------------------
    // If hardware handshake is used, the state of the RTS
    // and DTR signal can be set by software.
    if(bUseProtocol_l == RTS_CTS_PROTOCOL)
    {
        // RTS-Signal-Support was configured?
        #if(RTS_SIGNAL_SUPPORT != FALSE)
        {
            switch(pSioState_p->m_bRTS)
            {
                case SIGNAL_SET_INACTIVE:
    
                    CLR_RTS;// clear RTS signal
                    break;
        
                case SIGNAL_SET_ACTIVE:
        
                    SET_RTS;// set RTS signal
                    break;  
    
                case SIGNAL_NOT_CHANGE: 
                    break;
    
            }// End switch                                    

        }// End #if 
        
        #else
        {   // RTS-Signal-Support was not set, return an Error.
            pSioState_p->m_bErrCode = kSioDrvStatRtsSetError;
            bRetcode = FALSE;
        }// End #else
        #endif
    
        //---------------------------------------------------
        // DTR-Signal-Support was configured?
        #if(DTR_SIGNAL_SUPPORT != FALSE)
        {
            switch(pSioState_p->m_bDTR)
            {
                case SIGNAL_SET_INACTIVE:
    
                       CLR_DTR;// clear DTR signal
                       break;
        
                case SIGNAL_SET_ACTIVE:
    
                       SET_DTR;// set DTR signal
                       break;  
    
                case SIGNAL_NOT_CHANGE: break;
                
    
            }// End switch                                    
        }
        #else
        {   // DTR-Signal-Support was not set, return an Error.
            pSioState_p->m_bErrCode = kSioDrvStatDtrSetError;
            bRetcode = FALSE;
        }
        #endif     

        goto Exit;
    }//End if       

    //---------------------------------------------------------
    // If software protocol XON/XOFF or no protocoll is used, the state of
    // RTS and CTS signal can not be changed.
    if((bUseProtocol_l == XON_XOFF_PROTOCOL) || (bUseProtocol_l == NO_PROTOCOL))
    {
        if(pSioState_p->m_bRTS != SIGNAL_NOT_CHANGE)
        {
            pSioState_p->m_bErrCode = kSioDrvStatRtsNotSupported;
            bRetcode = FALSE;
            goto Exit;
        }// End if
    
        if(pSioState_p->m_bDTR != SIGNAL_NOT_CHANGE)
        {
            pSioState_p->m_bErrCode = kSioDrvStatDtrNotSupported;
            bRetcode = FALSE;
            goto Exit;
        }// End if
        
        // The actual signal state of RTS or DTR not change
        else
        {   //no error
            goto Exit;
        }
    }// End if
    
Exit:
    return (bRetcode);   
}


//---------------------------------------------------------------------------
//
// Function:     SioTxFifoPutByte
//
// Description:  This function is called by application to put
//               a character to FIFO of serial interface. After
//               character has been written, the transmission is triggered.
//
//
// Parameters:   character
//
// 
// Returns:      Errorcode
//               TRUE:  character sent
//               FALSE: character not sent
//
//
// State:        o.k. 2004/02/16 -ct
//
//---------------------------------------------------------------------------

BOOL PUBLIC SioTxFifoPutByte(char bChar_p)
{
BOOL fRes;
BOOL fBreak;

    // Unabhngig vom bertragungsmode wird stets
    // das Zeichen im Sendepuffer abgelegt und die
    // TX-Task freigegeben. 
    fRes = SioRBufTxWrite (bChar_p);

    do 
    {
       // TX-Task freigegeben, um Sendung auszufhren
       SioTriggerTx();

       //------------------------------------------------------------
       // Sendefunktion nicht blockierend
       #if(USE_BLOCKED_MODE != TRUE)
           fBreak = FALSE;
       #else
       //------------------------------------------------------------
       // Im blockierenden Mode wird hier gewartet,
       // bis das Zeichen bertragen wurde.
           fBreak = (SioRBufTxDataSize() != 0)?TRUE:FALSE;
       #endif

    } while (fBreak);

    return (fRes);
}


//---------------------------------------------------------------------------
//
// Function:     SioRxFifoGetByte
//
// Description:  This function is called by application to get
//               a character from FIFO of serial interface. After
//               character has been read, the receive interrupt chain
//               is triggered again.
//
//
// Parameters:   pointer to memory for storing character
//
// 
// Returns:      error code
//
//
// State:        o.k. 2004/02/16 -ct
//
//---------------------------------------------------------------------------

BOOL PUBLIC SioRxFifoGetByte(char FAR* pbValue_p)
{
    // Es wird versucht ein Zeichen zu lesen. Ist 
    // der Puffer leer, so wird durch Setzen der Steuerleitung RTS
    // dem Sender signalisiert, das weitere Zeichen empfangen werden knnen. 
    if (!SioRBufRxRead (pbValue_p))
    {
        // Die Funktion prft, ob gegebenenfalls das
        // Handshake RTS oder XON zu senden ist.
        SioTriggerRx();

        return FALSE;
    }

    return TRUE;
}

//---------------------------------------------------------------------------
//
// Function:     SioTxFifoDataSize
//
// Description:  This function is called by application to get
//               the size of data in TX-FIFO of serial interface. 
//
//
// Parameters:   pointer to memory for storing size
//
// 
// Returns:      error code
//
//
// State:        tested (18.06.2004, A.S.)
//
//---------------------------------------------------------------------------

BOOL PUBLIC SioTxFifoDataSize(DWORD FAR* pdwSize_p)
{
    *pdwSize_p = SioRBufTxDataSize();

    return TRUE;
}


//---------------------------------------------------------------------------
//
// Function:     SioRxFifoDataSize
//
// Description:  This function is called by application to get
//               the size of data in RX-FIFO of serial interface. 
//
//
// Parameters:   pointer to memory for storing size
//
// 
// Returns:      error code
//
//
// State:        tested (18.06.2004, A.S.)
//
//---------------------------------------------------------------------------

BOOL PUBLIC SioRxFifoDataSize(DWORD FAR* pdwSize_p)
{
    *pdwSize_p = SioRBufRxDataSize();

    return TRUE;
}

//=========================================================================//
//                                                                         //
//          P R I V A T E   F U N C T I O N S                              //
//                                                                         //
//=========================================================================//

//---------------------------------------------------------------------------
//
// Function:     SioTxIsrHandler
//
// Description:  Interrupt-Handler for tx channel
//
//
// Parameters:   none
// 
//
// Returns:      none
//
//
// State:        o.k. 2004/02/16 -ct
//
//---------------------------------------------------------------------------
//
// Erstes Zeichen senden: Sende-Task nicht aktiv, Sendefunktion gibt Sende-Task frei
// Zeichen senden: ISR prft, ob Zeichen vorhanden und sendet diese. Ist Sendepuffer
// leer, dann Sende-Task sperren.
// Receiver nicht bereit: Senden kann nicht erfolgen, da Empfnger nicht bereit.
// Die Sendetask wird gesperrt. Innerhalb der Applikation mu zyklisch eine Funktion 
// gerufen werden, die die Sendetask frei gibt, um ein fortfahren der Sendung zu prfen.

static void SioTxIsrHandler (void)
{
char bTxChar;

    // HW-Handshake: check if receiver is ready
    // XON-XOFF:     check if state == XOFF
    if ( !CTS_STATE )
    {
       // Der Empfnger ist nicht bereit, weitere Zeichen
       // zu empfangen. Die TX-Task bleibt im Running, wird 
       // aber zurckgestellt, bis ein Ereignis eintrifft CTS = TRUE). 
       CLRSTAT();
       return;
    }

    // XON-XOFF: check to send XOFF requested
    if ( fXOFF_l == TRUE )
    {
       // Der Sender mu ein XOFF einfgen, um das weitere
       // Senden der Gegenstelle zu unterbrechen.
       SioHwPutChar(XOFF);
       // RTS wieder auf TRUE setzen, um zu verhindern,
       // das stndig XOFF gesendet wird. fXON auf TRUE
       // Setzen um zu signalisieren, da bei leerem Puffer
       // XON gesendet werden kann.
       fXOFF_l = FALSE; 
       fXON_l  = TRUE;

       // Die TX-Task bleibt weiter aktiv, da ja nur in den 
       // laufenden Sendestrom ein XOFF eingefgt wurde.
       SETSTAT();
       return;
    }

    // Sind Zeichen im Sendepuffer?
    if (SioRBufTxRead(&bTxChar))
    {
       // Zeichen sind im Sendepuffer vorhanden.
       SioHwPutChar(bTxChar);
       // Die TX-Task bleibt weiter aktiv.
       SETSTAT();
    }
    else
    {   
       // Der Sendepuffer ist leer, daher die Tx-Task 
       // sperren.
       CLRSTAT();
    }
}

//---------------------------------------------------------------------------
//
// Function:     SioRxIsrHandler
//
// Description:  Interrupt-Handler for rx channel
//
//
// Parameters:   none
// 
//
// Returns:      none
//
//
// State:        o.k. 2004/02/16 -ct
//
//---------------------------------------------------------------------------

static void SioRxIsrHandler (void)
{
char bRxChar;

    bRxChar=(char)SioHwGetChar();

    // XON-XOFF-Steuerung beachten?
    if (bUseProtocol_l == XON_XOFF_PROTOCOL)
    {
       if (bRxChar == XOFF)
       {
          // Zeichen nicht im Empfangspuffer ablegen,
          // nur Status sichern, um weiteren Sendeverlauf zu steuern
          CLR_CTS;
          return;
       }

       if (bRxChar == XON)
       {
          // Zeichen nicht Empfangspuffer ablegen,
          // nur Status sichern, um weiteren Sendeverlauf zu steuern
          SET_CTS;
          return;
       }     
    }

    // put received byte to FIFO
    SioRBufRxWrite(bRxChar);
    
    // get byte from register
    if (SioRBufRxDataSize() > SER_RXFIFO_STOP_LEVEL)
    {
        // HW-Handshake: RTS lschen und damit Sender sperren
        if (bUseProtocol_l == RTS_CTS_PROTOCOL)
        {
           CLR_RTS;
        }

        // XON-XOFF: Flag setzen, um XOFF in der Sende-Task zu senden
        if (bUseProtocol_l == XON_XOFF_PROTOCOL)
        {
           fXOFF_l = TRUE;
           // Ist die Sende-Task gerade nicht aktiv, so 
           // Task signalisieren, um die bertragung zu starten.
           SioTriggerTx();
        }
    }
}

//---------------------------------------------------------------------------
//
// Function:     SioErrIsrHandler
//
// Description:  Handler fr den Error-Interrupt
//
//
// Parameters:   none
// 
//
// Returns:      none
//
//
// State:        
//
//---------------------------------------------------------------------------
/*
static void SioErrIsrHandler (void)
{

}
*/
//---------------------------------------------------------------------------
//
// Function:     SioTriggerRx
//
// Description:  This function is called by serial driver to trigger
//               data receive just do handshaking.
//
//
// Parameters:   none
//
// 
// Returns:      none
//
//
// State:        o.k. 2004/02/16 -ct
//
//---------------------------------------------------------------------------

static void SioTriggerRx(void)
{
    if (SioRBufRxDataSize() == 0)
    {
        if (bUseProtocol_l == RTS_CTS_PROTOCOL)
        {
           SET_RTS;        // weitere Zeichen anfordern, da Puffer leer
        }
        // XON senden?
        if (fXON_l  && (bUseProtocol_l == XON_XOFF_PROTOCOL))
        {
           fXON_l = FALSE;
           SioTxFifoPutByte(XON);
        }
    }
}

//---------------------------------------------------------------------------
//
// Function:     SioTriggerTx
//
// Description:  This function is called by serial driver to trigger
//               data sending.
//
//
// Parameters:   none
//
// 
// Returns:      none
//
//
// State:        o.k. 2004/02/16 -ct
//
//---------------------------------------------------------------------------

static void SioTriggerTx(void)
{
    // Die TX-Task freigeben.
    SETSTAT();
}

/***************************************************************************/
/*                                                                         */
/*                                                                         */
/*          C L A S S  <SerialDrv Hardware interface>                      */
/*                                                                         */
/*                                                                         */
/***************************************************************************/
//
// Description:
// Die Klasse stellt Funktionen zur Verfgung fr den Zugriff auf die
// Hardware der seriellen Schnittstelle. Diese Funktionen werden nur intern
// verwendet, um den Zugriff auf die Hardware im Generic-Treiber zu kapseln.
//
// Zeichen senden:
// Das Senden von Zeichen erfolgt ausschlielich in der TX-ISR. Beim Senden 
// von Zeichen mit Hardware-Protokoll wird vor dem bertragen des nchsten 
// Zeichens geprft, ob der Empfnger auch bereit ist, Zeichen zu empfangen. 
// Wurde das Senden gesperrt, so mu zyklisch durch die Applikation der
// Sendekanal getriggert werden, um bei wieder empfangsbereitem Empfnger
// mit der bertragung fortzufahren. Da sowohl das Senden von Zeichen als 
// auch das Stoppen der bertragung innerhalb der TX-ISR realisiert ist, ist
// ein Signalisierung aus zwei Richtungen erforderlich.
// Zum einen erfolgt eine Signalisierung durch die Hardware, wenn ein Zeichen
// vollstndig bertragen wurde und der Sendepuffer leer ist. In diesem 
// Betriebsmodus darf jedoch die ISR nicht durch die Applikation getriggert 
// werden (Die ISR geht davon aus, da das nchste Zeichen im Sendepuffer
// zu hinterlegen ist). Nach vollstndiger bertragung aller Zeichen oder
// bei Unterbrechen der bertragung durch den Empfnger wird das Signalisieren 
// durch die Hardware gesperrt. Der Betriebsmodus wird umgeschaltet, nun
// kann aus der Applikation die ISR signalisiert werden, um den Status der 
// Handshake-Leitungen zu prfen. Beide Betriebsmodies mssen sich stets
// ausschlieen. Ein Umschalten des Betriebsmodus darf nur innerhalb der ISR
// erfolgen.
//
/***************************************************************************/

//=========================================================================//
//                                                                         //
//          P R I V A T E   D E F I N I T I O N S                          //
//                                                                         //
//=========================================================================//

//---------------------------------------------------------------------------
// const defines
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
// local types
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
// lokale vars
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
// local function prototypes
//---------------------------------------------------------------------------

//=========================================================================//
//                                                                         //
//          P U B L I C   F U N C T I O N S                                //
//                                                                         //
//=========================================================================//

//---------------------------------------------------------------------------
//
// Function:     SioHwInit
//
// Description:  The function configures the serial interface.
//
//
// Parameters:   parameter
//
// 
// Returns:      error code
//
//
// State:        o.k. 2004/02/16 -ct
//
//---------------------------------------------------------------------------
 
static BOOL SioHwInit(tSioCfgParam FAR* pSioCfgParam_p)
{
WORD  wCONCfg = 0x0000;
BOOL  fRetcode = TRUE;


    //  Configuration of the used ASCx Port Pins:
    //  - P3.10 is used for ASCx Transmit Data Output (TxDA0)
    //  - P3.11 is used for ASCx Receive Data Input (RxDA0)
    COM_PORT  |= COM_PORT_MASK;
    ALTSEL0P3 |= COM_PORT_MASK;
    COM_DPORT |= COM_PORT_MASK; 
    
    //------------------------------------------------
    switch (pSioCfgParam_p->m_dwBaudrate)
    {

            //  - fractional divider as prescaler for baud rate timer is used
            case 9600L :    //  - required baud rate = 9,600 kbaud
                            //  - real baud rate     = 9,600 kbaud
                            //  - deviation          = 0,001 %
                            ASCx_FDV       =  0x0074;      // load ASCx fractional divider register
                            ASCx_BG        =  0x003A;     // load ASCx baud rate time reload register
                            break;
            
            case 19200L :   //  - required baud rate = 19,200 kbaud
                            //  - real baud rate     = 19,200 kbaud
                            //  - deviation          = 0,001 %
                            
                            ASCx_FDV       =  0x00E8;      // load ASCx fractional divider register
                            ASCx_BG        =  0x003A;     // load ASCx baud rate time reload register
                            break;
                    
            case 38400L :   //  - required baud rate = 38,400 kbaud
                            //  - real baud rate     = 38,400 kbaud
                            //  - deviation          = 0,001 %
                            ASCx_FDV       =  0x01D0;     // load ASCx fractional divider register
                            ASCx_BG        =  0x003A;     // load ASCx baud rate time reload register
                            break;
                        
            case 57600L :   //  - required baud rate = 57,600 kbaud
                            //  - real baud rate     = 57,592 kbaud
                            //  - deviation          = -0,014 %
                            ASCx_FDV       =  0x01CC;      // load ASCx fractional divider register
                            ASCx_BG        =  0x0026;      // load ASCx baud rate time reload register
                            break;                    
                    
            case 115200L :  //  - required baud rate = 115,200 kbaud
                            //  - real baud rate     = 115,177 kbaud
                            //  - deviation          = -0,020 %
                            ASCx_FDV       =  0x0191;      // load ASCx fractional divider register
                            ASCx_BG        =  0x0010;      // load ASCx baud rate time reload register
                            break;

            default:
                            // set error code for baudrate not supported   
                            pSioCfgParam_p->m_bErrCode=kSioDrvInitBaudrateNotSupported; 
                            fRetcode = FALSE; 
                            goto Exit;            

    }// End switch
    
    //------------------------------------------------
    switch(pSioCfgParam_p->m_bDataBits)
    {
        case 7:
             wCONCfg |= 0x0003;//set seven databits and parity              
             break;

        case 8:  
             if(pSioCfgParam_p->m_bParity)
             {
                wCONCfg |= 0x0007;  //set eight databits async, with parity
             }// End if
             else
             {
                wCONCfg &= ~0x0007;
                wCONCfg |= 0x0001; // set 8 databits async, without parity
             }// End else
             break;

        default:        
            // set error code for databits not supported
            pSioCfgParam_p->m_bErrCode = kSioDrvInitNumOfDataBitsNotSupported; 
            fRetcode = FALSE;                 
            goto Exit;     
   
   }// End switch

   //------------------------------------------------
   switch(pSioCfgParam_p->m_bParity)
   {
       
        case PARITY_NONE: 
            wCONCfg &= ~0x0020; //disable parity check
            break;
    
        case PARITY_ODD:
            wCONCfg |= 0x1020;  //enable odd parity
            break;  
    
        case PARITY_EVEN:
            wCONCfg |= 0x0020;
            wCONCfg &= ~0x1000; //enable even parity
            break;
    
        default:
            //set error code for parity not supported
            pSioCfgParam_p->m_bErrCode = kSioDrvInitSelParityNotSupported; 
            fRetcode = FALSE; 
            goto Exit;
    }// End switch

    //------------------------------------------------
    switch(pSioCfgParam_p->m_bStopBits)
    {
        case 1:
            wCONCfg &= ~0x0008;  //set one stop bit
            break;

        case 2:
            wCONCfg |= 0x0008;   //set two stopbits
            break;

        default:
            //set error code for stopbits not supported
            pSioCfgParam_p->m_bErrCode = kSioDrvInitNumOfStopBitsNotSupported; 
            fRetcode = FALSE;
            goto Exit;

    }// End switch

    //------------------------------------------------
    switch(pSioCfgParam_p->m_bProtocol)
    {
        #if (HW_HANDSHAKE_SUPPORT != FALSE)
          case 2:
              bUseProtocol_l = RTS_CTS_PROTOCOL; // Hardware handshake is used
              break;
        #else
          // kein Protokoll 
          case 0: 
              bUseProtocol_l = NO_PROTOCOL; // no protocol is used           
              break;

          case 1:
              bUseProtocol_l = XON_XOFF_PROTOCOL; // XON/XOFF protocol is used               
              break;
        #endif
     

        default:
            //set error code for protocol not supported
            pSioCfgParam_p->m_bErrCode = kSioDrvInitSelProtocolNotSupported; 
            fRetcode = FALSE; 
            goto Exit;
    }

    //------------------------------------------------    

    //  Configuration of the ASCx Operation Mode:
    //  - receiver is enabled
    //  - fractional divider is enabled
    wCONCfg |= 0x8811;     
    ASCx_CON       = wCONCfg;  // set ASCx-CON register

    //  Configuration of the used ASCx Interrupts:
    ASCx_TIC = (ASCx_TXINT_GPX<<8) | (ASCx_TXINT_ILVL<<2) | ASCx_TXINT_GLVL;   // set priority
    ASCx_TIC_IE = 1;     // enable interrupts
 
    ASCx_RIC = (ASCx_RXINT_GPX<<8) | (ASCx_RXINT_ILVL<<2) | ASCx_RXINT_GLVL;   // set RX priority
    ASCx_RIC_IE=1;       // enable RX interrupts 

    ASCx_EIC = 0;        // disable error interrupts

Exit:

    return (fRetcode);

}

//---------------------------------------------------------------------------
//
// Function:     SioHwClose
//
// Description:  The function disabled the serial interface and 
//               stops the interrupt processing.
//
//
// Parameters:   none
//
// 
// Returns:      TRUE
//
//
// State:        tested (18.06.2004, A.S.)
//
//---------------------------------------------------------------------------

static BOOL SioHwClose(void)
{
    // deinitialise ASCx
    ASCx_TIC=0x0000;
    ASCx_RIC=0x0000;
    ASCx_CON=0x0000;
    return TRUE;
}


//---------------------------------------------------------------------------
//
// Function:     SioHwPutChar
//
// Description:  The function sends one character.
//
//
// Parameters:   character
//
// 
// Returns:      none
//
//
// State:        o.k. 2004/02/16 -ct
//
//---------------------------------------------------------------------------

static void SioHwPutChar(char Char)
{
    ASCx_TBUF = Char;
}

//---------------------------------------------------------------------------
//
// Function:     SioHwGetChar
//
// Description:  The function reads one character from serial interface.
//
//
// Parameters:   none
//
// 
// Returns:      character
//
//
// State:        o.k. 2004/02/16 -ct 
//
//---------------------------------------------------------------------------

static char SioHwGetChar(void)
{
    return ((char) ASCx_RBUF);
}

//---------------------------------------------------------------------------
//
// Function:     SioHwGetErrorState
//
// Description:  The function determines the error state of serial interface.
//
//
// Parameters:   none
//
// 
// Returns:      Error flags
//               Bit0: Overrun Error
//               Bit1: Parity Error
//               Bit2: Framing Error
//
//
// State:        
//
//---------------------------------------------------------------------------

static BYTE SioHwGetErrorState(void)
{
BYTE bStatus = 0;
WORD wSCON;

    wSCON  = ASCx_CON;
    ASCx_CON &= ~0x0700; // Fehlerstatus lschen
    
    // Overrun error occured?
    bStatus |= wSCON&0x0400?0x01:0x00; 

    // Parity error occured?
    bStatus |= wSCON&0x0100?0x02:0x00; 

    // Framing error occured?
    bStatus |= wSCON&0x0200?0x04:0x00; 

    return (bStatus);
}

//---------------------------------------------------------------------------
//
// Function:     SioHwSignalTxIsr
//
// Description:  The function sends a signal to tx task.
//
//
// Parameters:   none
//
// 
// Returns:      none
//
//
// State:        o.k. 2004/02/16 -ct
//
//---------------------------------------------------------------------------

static void SioHwSignalTxIsr(void)
{
    ASCx_TIC_IE = 1;
}


//---------------------------------------------------------------------------
//
// Function:     SioHwDisableTxIsr
//
// Description:  The function disables tx task.
//
//
// Parameters:   none
//
// 
// Returns:      none
//
//
// State:        o.k. 2004/02/16 -ct
//
//---------------------------------------------------------------------------

static void SioHwDisableTxIsr(void)
{
    ASCx_TIC_IE = 0;
    ASCx_TIC_IR = 1;
}

//---------------------------------------------------------------------------
//
// Function:     SioHwTxIsr
//
// Description:  Interrupt service function for tx channel
//
//
// Parameters:   none
//
// 
// Returns:      none
//
//
// State:        o.k. 2004/02/16 -ct
//
//---------------------------------------------------------------------------

static void SioHwTxIsr (void) interrupt ASCx_TINT
{
    SioTxIsrHandler();    
}


//---------------------------------------------------------------------------
//
// Function:     SioHwRxIsr
//
// Description:  Interrupt service function for rx channel
//
//
// Parameters:   none
//
// 
// Returns:      none
//
//
// State:        o.k. 2004/02/16 -ct
//
//---------------------------------------------------------------------------

static void SioHwRxIsr (void) interrupt ASCx_RINT
{
    SioRxIsrHandler();    
}


//---------------------------------------------------------------------------
//
// Function:     SioHwErrIsr
//
// Description:  Interrupt service function for error interrupt
//
//
// Parameters:   none
//
// 
// Returns:      none
//
//
// State:        
//
//---------------------------------------------------------------------------
/*
static void SioHwErrIsr (void) interrupt ASCx_EINT
{
    SioRxIsrHandler();    
}
*/

/***************************************************************************/
/*                                                                         */
/*                                                                         */
/*          C L A S S  <Ring buffer>                                       */
/*                                                                         */
/*                                                                         */
/***************************************************************************/
//
// Description:
// Die Ringpuffer-Verwaltung ist so implementiert, da fr den Zugriff
// auf den Puffer keine Sperre fr den komplementren Zugriff auf den Puffer
// aus einer hherpriorisierten Task notwendig ist. D.h. Eine Lesezugriff
// kann durchaus durch einen Schreibzugriff unterbrochen werden. Es ist 
// zu verhindern, da ein Lesezugriff durch einen Lesezugriff auf den 
// gleichen Puffer aus einer hherpriorisierten Task erfolgt. Fr die 
// korrekte Funktion ist weiterhin wichtig, da das Lesen oder Schreiben eines
// der Puffer-Indizes stets in einer nichtunterbrechbaren Sequenz erfolgt
// (Ausfhrung in einem Maschinenbefehl). Da das von Maschine zu Maschine
// unterschiedlich ist, wurden die Datentypen fr die Indizes ber Konstanten
// definiert, um maschinenabhngig den Zugriff steuern zu knnen und zustzlich 
// (vorsorglich, falls das aus technischen Grnden nicht anders mglich ist, da
// z.B. der Puffer sehr gro sein mu) der Zugriff durch eine ATOMIC/ENDATOMIC
// Anweisung eingerahmt. Diese Makros knnen durch Funktionen oder Anweisungen
// ersetzt werden, die dann whrend des Zugriffs die hherpriorisierte Task 
// sperren.
// 
//
//---------------------------------------------------------------------------


//---------------------------------------------------------------------------
// const defines
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
//  Target specific definitions
//---------------------------------------------------------------------------

#ifdef __C166__
    #define ATOMIC_START() _atomic_(0)
    #define ATOMIC_END()   _endatomic_() 
#else
    #define ATOMIC_START() 
    #define ATOMIC_END()   
#endif   


//---------------------------------------------------------------------------
// local types
//---------------------------------------------------------------------------
// Der Type fr den Buffer-Index mu machinenabhngig so gewhlt
// werden, da das Lesen oder Schreiben des Index nicht durch einen 
// Interrupt unterbrochen werden kann. Beim einem 16Bit-Controller 
// ist das in der Regel ein 16Bit-Zugriff, beim PC ab x386 ein 32Bit-
// Zugriff. Wird auf Grund der Puffergre einer grerer Typ bentigt,
// so sind die ATOMIC-Makros so anzupassen, da eine Unterbrechung
// whrend des Zugriffs nicht mglich ist.
#define tBufIdx WORD

//---------------------------------------------------------------------------
// lokale vars
//---------------------------------------------------------------------------

static BYTE     abRxRingBuff_l[SERIAL_RX_FIFO_SIZE];
static BYTE     abTxRingBuff_l[SERIAL_TX_FIFO_SIZE];
static tBufIdx  wRxBuffWrIdx_l;
static tBufIdx  wRxBuffRdIdx_l;
static tBufIdx  wTxBuffWrIdx_l;
static tBufIdx  wTxBuffRdIdx_l;


//---------------------------------------------------------------------------
// local function prototypes
//---------------------------------------------------------------------------

static BOOL SioRBufTxIsEmpty  (void);
static BOOL SioRBufRxIsEmpty  (void);


//=========================================================================//
//                                                                         //
//          P U B L I C   F U N C T I O N S                                //
//                                                                         //
//=========================================================================//

//---------------------------------------------------------------------------
//
// Function:     SioRBufInit
//
// Description:  The function initializes the rx and tx buffer.
//
//
// Parameters:   none
//
// 
// Returns:      none
//
//
// State:        o.k. 2004/02/16 -ct
//
//---------------------------------------------------------------------------

static void SioRBufInit (void)
{            
    SioRBufRxClear ();
    SioRBufTxClear ();
}

//---------------------------------------------------------------------------
//
// Function:     SioRBufRxClear
//
// Description:  The function clears the rx buffer.
//
//
// Parameters:   none
//
// 
// Returns:      none
//
//
// State:        o.k. 2004/02/16 -ct
//
//---------------------------------------------------------------------------

static void SioRBufRxClear (void)
{

    ATOMIC_START();
    {    
        wRxBuffWrIdx_l = 0;
        wRxBuffRdIdx_l = 0;
    }
    ATOMIC_END();
}

//---------------------------------------------------------------------------
//
// Function:     SioRBufTxClear
//
// Description:  The function clears the tx buffer.
//
//
// Parameters:   none
//
// 
// Returns:      none
//
//
// State:        o.k. 2004/02/16 -ct
//
//---------------------------------------------------------------------------

static void SioRBufTxClear (void)
{

    ATOMIC_START();
    {    
        wTxBuffWrIdx_l = 0;
        wTxBuffRdIdx_l = 0;
    }
    ATOMIC_END();
}


//---------------------------------------------------------------------------
//
// Function:     SioRBufTxWrite
//
// Description:  The function writes data to tx buffer.
//
//
// Parameters:   data byte
//
// 
// Returns:      error code
//               TRUE:  character has been written
//               FALSE: buffer full
//
//
// State:        o.k. 2004/02/16 -ct
//
//---------------------------------------------------------------------------

static BOOL SioRBufTxWrite (BYTE bData_p)
{
WORD  wNextBuffWrIdx;
WORD  wBuffRdIdx;
BOOL  fRes;


    // get buffer index for next write operation
    ATOMIC_START();
    {
       wNextBuffWrIdx = wTxBuffWrIdx_l;
    }
    ATOMIC_END();
    
    wNextBuffWrIdx += 1;
    if (wNextBuffWrIdx >= sizeof(abTxRingBuff_l))
    {
        wNextBuffWrIdx = 0;
    }


    // free buffer availabale (write index and read index not identical)?
    ATOMIC_START();
    {
       wBuffRdIdx = wTxBuffRdIdx_l;
    }
    ATOMIC_END();

    fRes = (wNextBuffWrIdx != wBuffRdIdx);

    if (fRes)
    {
        // free buffer available -> write data to buffer
        abTxRingBuff_l[wTxBuffWrIdx_l] = bData_p;

        ATOMIC_START();
        {
            wTxBuffWrIdx_l = wNextBuffWrIdx;
        }
        ATOMIC_END();
    }

    return (fRes);
}
//---------------------------------------------------------------------------
//
// Function:     SioRBufRxWrite
//
// Description:  The function writes data to rx buffer.
//
//
// Parameters:   data byte
//
// 
// Returns:      error code
//               TRUE:  character has been written
//               FALSE: buffer full
//
//
// State:        o.k. 2004/02/16 -ct
//
//---------------------------------------------------------------------------
static BOOL SioRBufRxWrite (BYTE bData_p)
{
WORD  wNextBuffWrIdx;
WORD  wBuffRrIdx;
BOOL  fRes;


    // get buffer index for next write operation
    ATOMIC_START();
    {
       wNextBuffWrIdx = wRxBuffWrIdx_l;
    }
    ATOMIC_END();

    wNextBuffWrIdx += 1;
    if (wNextBuffWrIdx >= sizeof(abRxRingBuff_l))
    {
        wNextBuffWrIdx = 0;
    }

    // free buffer availabale (write index and read index not identical)?
    ATOMIC_START();
    {
       wBuffRrIdx = wRxBuffRdIdx_l;
    }
    ATOMIC_END();

    fRes = (wNextBuffWrIdx != wBuffRrIdx);

    if (fRes)
    {
        // free buffer availabale -> write data to buffer
        abRxRingBuff_l[wRxBuffWrIdx_l] = bData_p;

        ATOMIC_START();
        {
           wRxBuffWrIdx_l = wNextBuffWrIdx;
        }
        ATOMIC_END();
    }

    return (fRes);
}


//---------------------------------------------------------------------------
//
// Function:     SioRBufTxRead
//
// Description:  The function reads data from tx buffer.
//
//
// Parameters:   pointer to storage data byte
//
// 
// Returns:      error code
//               TRUE:  character has been read
//               FALSE: buffer empty
//
//
// State:        o.k. 2004/02/16 -ct
//
//---------------------------------------------------------------------------

static BOOL SioRBufTxRead (BYTE FAR* pbData_p)
{

WORD  wNextBuffRdIdx;
BOOL  fRes;


    // Is at least one data entry available?
    fRes = SioRBufTxIsEmpty();

    if ( fRes )
    {
        // return the data from buffer
        *pbData_p = abTxRingBuff_l[wTxBuffRdIdx_l];

        // get buffer index for next read operation
        ATOMIC_START();
        {
           wNextBuffRdIdx = wTxBuffRdIdx_l;
        }
        ATOMIC_END();

        wNextBuffRdIdx += 1;
        if (wNextBuffRdIdx >= sizeof(abTxRingBuff_l))
        {
            wNextBuffRdIdx = 0;
        }

        ATOMIC_START();
        {
            wTxBuffRdIdx_l = wNextBuffRdIdx;
        }        
        ATOMIC_END();
    }

    return (fRes);
}

//---------------------------------------------------------------------------
//
// Function:     SioRBufRxRead
//
// Description:  The function reads data from rx buffer.
//
//
// Parameters:   pointer to storage data byte
//
// 
// Returns:      error code
//               TRUE:  character has been read
//               FALSE: buffer empty
//
//
// State:        o.k. 2004/02/16 -ct
//
//---------------------------------------------------------------------------

static BOOL SioRBufRxRead (BYTE FAR* pbData_p)
{

WORD  wNextBuffRdIdx;
BOOL  fRes;


    // Is at least one data entry available?
    fRes = SioRBufRxIsEmpty();

    // buffer is not empty
    if ( fRes )
    {
        // return the data from buffer
        *pbData_p = abRxRingBuff_l[wRxBuffRdIdx_l];

        // get buffer index for next read operation
        ATOMIC_START();
        {
           wNextBuffRdIdx = wRxBuffRdIdx_l;
        }
        ATOMIC_END();

        wNextBuffRdIdx += 1;
        if (wNextBuffRdIdx >= sizeof(abRxRingBuff_l))
        {
            wNextBuffRdIdx = 0;
        }

        ATOMIC_START();
        {
            wRxBuffRdIdx_l = wNextBuffRdIdx;
        }        
        ATOMIC_END();
    }

    return (fRes);
}

//---------------------------------------------------------------------------
//
// Function:     SioRBufRxDataSize
//
// Description:  Function gets the number of data bytes in Rx-Buffer.
//
//
// Parameters:   none
//
// 
// Returns:      data size of rx buffer
//
//
// State:        tested (18.06.2004 A.S.)
//
//---------------------------------------------------------------------------

static WORD SioRBufRxDataSize (void)
{
WORD  wIdxDiff;

    // get the difference of write index and read index (in a safe section!)
    ATOMIC_START();
    {
        wIdxDiff = wRxBuffWrIdx_l - wRxBuffRdIdx_l;
    }
    ATOMIC_END();

    return (wIdxDiff);
}

//---------------------------------------------------------------------------
//
// Function:     SioRBufTxDataSize
//
// Description:  Function gets the number of data bytes in Tx-Buffer.
//
//
// Parameters:   none
//
// 
// Returns:      data size of tx buffer
//
//
// State:        tested (18.06.2004 A.S.)
//
//---------------------------------------------------------------------------

static WORD SioRBufTxDataSize (void)
{
WORD  wIdxDiff;

    // get the difference of write index and read index (in a safe section!)
    ATOMIC_START();
    {
        wIdxDiff = wTxBuffWrIdx_l - wTxBuffRdIdx_l;
    }
    ATOMIC_END();

    return (wIdxDiff);
}

//=========================================================================//
//                                                                         //
//          P R I V A T E   F U N C T I O N S                              //
//                                                                         //
//=========================================================================//

//---------------------------------------------------------------------------
//
// Function:     SioRBufRxIsEmpty
//
// Description:  Function checks if any data is available in Rx-Buffer.
//
//
// Parameters:   none
//
// 
// Returns:      TRUE:  data available
//               FALSE: no data available
//
//
// State:        o.k. 2004/02/16 -ct
//
//---------------------------------------------------------------------------

static BOOL SioRBufRxIsEmpty (void)
{
BOOL  fRes;

    // is at least one data entry available
    // (are write index and read index identical)?
    fRes = (SioRBufRxDataSize() == 0)?FALSE:TRUE;

    return (fRes);
}

//---------------------------------------------------------------------------
//
// Function:     SioRBufTxIsEmpty
//
// Description:  Function checks if any data is available in Tx-Buffer.
//
//
// Parameters:   none
//
// 
// Returns:      TRUE:  data available
//               FALSE: no data available
//
//
// State:        o.k. 2004/02/16 -ct
//
//---------------------------------------------------------------------------

static BOOL SioRBufTxIsEmpty (void)
{
BOOL  fRes;

    // is at least one data entry available
    // (are write index and read index identical)?
    fRes = (SioRBufTxDataSize() == 0)?FALSE:TRUE;

    return (fRes);
}


//EOF
