/****************************************************************************

  (c) SYSTEC electronic GmbH, D-07973 Greiz, August-Bebel-Str. 29
      www.systec-electronic.de

  Project:      Command interpreter

  Description:  Interface to standard IO-terminal

  -------------------------------------------------------------------------

                $RCSfile: CmdIntTgt.c,v $
                
                $Author: A.Suess $
                
                $Revision: 1.1.1.1 $  $Date: 2004/09/16 11:56:05 $
                
                $State: Exp $
                
                Build Environment:
                KEIL uVision 2

  -------------------------------------------------------------------------

  Revision History:

  2004/01/12 -as:   implementation of target specific functions for
                    character transfer                   


****************************************************************************/

#include "generic/global.h"
#include "../../../include/siodrv.h"

/***************************************************************************/
/*                                                                         */
/*                                                                         */
/*          G L O B A L   D E F I N I T I O N S                            */
/*                                                                         */
/*                                                                         */
/***************************************************************************/

//---------------------------------------------------------------------------
// const defines
//---------------------------------------------------------------------------

// What instance of serial driver should be used?
// 0 --> ASC0
// 1 --> ASC1
// 2 --> external UART
#define SERIAL_INSTANCE 0

//---------------------------------------------------------------------------
// local types
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
// modul globale vars
//---------------------------------------------------------------------------

#if defined SERIAL_INSTANCE==2

// parameter for serial interface
static CONST tSioCfgParam SioCfgParam_l =
{
    kSioDrvInitOk,      // Error Code
    57600L,             // Baudrate
    8,                  // Databits
    PARITY_NONE,        // Parity
    1,                  // Stopbits
    RTS_CTS_PROTOCOL//NO_PROTOCOL         // Protocol
};

#else 
// parameter for serial interface
static CONST tSioCfgParam SioCfgParam_l =
{
    kSioDrvInitOk,      // Error Code
    115200L,            // Baudrate
    8,                  // Databits
    PARITY_NONE,        // Parity
    1,                  // Stopbits
    NO_PROTOCOL         // Protocol
};
#endif 
//---------------------------------------------------------------------------
// local function prototypes
//---------------------------------------------------------------------------

/***************************************************************************/
/*                                                                         */
/*                                                                         */
/*          C L A S S  <Standard character functions>                      */
/*                                                                         */
/*                                                                         */
/***************************************************************************/
//
// Description:
// Diese Klasse kapselt die Anbindung der serielle Schnittstelle. Fr den
// Kommandointerpreter werden nur die beiden Standard-Funktionen putchar
// und _getkey bentigt. Diese beiden Funktionen verzweigen hier auf den
// Standard-Sio-Treiber (Instanz 0), knnten aber genausogut auf Funktionen
// fr eine externe UART umgelenkt werden. Dieser Teil ist in der Regel 
// hardwarespezifisch und sollte daher in einem separaten File und target-
// spezifischen Verzeichnis liegen.
//
/***************************************************************************/


//=========================================================================//
//                                                                         //
//          P R I V A T E   D E F I N I T I O N S                          //
//                                                                         //
//=========================================================================//

//---------------------------------------------------------------------------
// const defines
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
// local types
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
// lokale vars
//---------------------------------------------------------------------------

static tSerialInst FAR* pSioInst_l;

//---------------------------------------------------------------------------
// local function prototypes
//---------------------------------------------------------------------------

//=========================================================================//
//                                                                         //
//          P U B L I C   F U N C T I O N S                                //
//                                                                         //
//=========================================================================//

//---------------------------------------------------------------------------
//
// Function:     CmtIntTgtInitSioDrv
//
// Description:  This function init. the SIO driver. 
//
// Parameters:   keine
// 
// Returns:      none
//
// State:        
//
//---------------------------------------------------------------------------

void PUBLIC CmtIntTgtInitSioDrv (void)
{
    SioInitInstData();

    // get pointer of the SioDrv instance
    pSioInst_l = SioGetInstData(SERIAL_INSTANCE);

    pSioInst_l->m_pfnSerialInit(&SioCfgParam_l);
}


//---------------------------------------------------------------------------
//
// Function:     CmdIntTgtGetChar
//
// Description:  This function checks for a received char in the buffer 
//               of the ASC0 and get it back.
//
// Parameters:   keine
// 
// Returns:      > 0    ==> character
//               = 0    ==> no character
//
// State:        
//
//---------------------------------------------------------------------------

BYTE PUBLIC CmdIntTgtGetChar (void)
{
char cZeichen;

    if( pSioInst_l->m_pfnGetByte((BYTE FAR*)&cZeichen) )
    {   // One character was received, return this character.
        return (cZeichen);    
    }
    else
    {  // No character was received.
       return (0);
    }
}

//---------------------------------------------------------------------------
//
// Function:     CmdIntTgtPutChar
//
// Description:  This function sends a char to standard device. 
//
// Parameters:   character
// 
// Returns:      none
//               
//
// State:        
//
//---------------------------------------------------------------------------

char PUBLIC CmdIntTgtPutChar (char cZeichen)
{

    pSioInst_l->m_pfnPutByte(cZeichen);

    return (cZeichen);
}

//---------------------------------------------------------------------------
//
// Function:     putchar
//
// Description:  This function sends a char to standard device. 
//
// Parameters:   character
// 
// Returns:      character
//               
//
// State:        
//
//---------------------------------------------------------------------------

signed char putchar (signed char cZeichen)  
{
DWORD dwBufSize;

    if (cZeichen == '\n')
    {
           pSioInst_l->m_pfnPutByte('\r');

           do
           {
              pSioInst_l->m_pfnTxFifoDataSize(&dwBufSize);

           } while (dwBufSize);
    }

    pSioInst_l->m_pfnPutByte(cZeichen);

    do
    {
       pSioInst_l->m_pfnTxFifoDataSize(&dwBufSize);

    } while (dwBufSize);

    return (cZeichen);
}


