/****************************************************************************

  (c) SYSTEC electronic GmbH, D-07973 Greiz, August-Bebel-Str. 29
      www.systec-electronic.de

  Project:      PLCmodule C14

  Description:  serial interface driver

  -------------------------------------------------------------------------

                $RCSfile: SioDrv.h,v $
                
                $Author: A.Suess $
                
                $Revision: 1.1.1.1 $  $Date: 2004/09/16 11:56:05 $
                
                $State: Exp $
                
                Build Environment:
                KEIL uVision 2

  -------------------------------------------------------------------------

  Revision History:

  2003/07/07 -avc:   Start der Implementierung

****************************************************************************/

#ifndef _SERIAL_H_
#define _SERIAL_H_


#define SIGNAL_NOT_CHANGE       0xFF    // (-1)
#define SIGNAL_SET_INACTIVE     0x0     // ( 0)
#define SIGNAL_SET_ACTIVE       0x1     // ( 1)
#define SIGNAL_NOT_SUPPORTED    0xFF    

#define NO_PROTOCOL             0       // no protocol is used  
#define XON_XOFF_PROTOCOL       1       // XON/XOFF protocol is used  
#define RTS_CTS_PROTOCOL        2       // RTS/CTS protocol is used  

#define PARITY_NONE             0
#define PARITY_ODD              1
#define PARITY_EVEN             2

#define CLR_NOT_CHANGE          0x0
#define CLR_RX_BUFFER           0x1
#define CLR_TX_BUFFER           0x2
#define CLR_RX_TX_BUFFER        0x3

#define SIODRV_INACTIVE         0x00
#define SIODRV_INIT             0x01
#define SIODRV_ECHO_SUPPORTED   0x80



//---------------------------------------------------------------------------
// typedefs
//---------------------------------------------------------------------------

// Error code for function SerInit
typedef enum
{
    kSioDrvInitOk                        =  0,
    kSioDrvInitPortNotSupported          =  2,
    kSioDrvInitBaudrateNotSupported      =  4,
    kSioDrvInitNumOfDataBitsNotSupported =  8,
    kSioDrvInitSelParityNotSupported     = 16,
    kSioDrvInitNumOfStopBitsNotSupported = 32,
    kSioDrvInitSelProtocolNotSupported   = 64

} tSioDrvInitErrCode; 



// Error code for function GetState, SetState
typedef enum
{
    kSioDrvStatOk                        =  0,
    kSioDrvStatPortNotSupported          =  2,
    kSioDrvStatRtsSetError               =  4,
    kSioDrvStatDtrSetError               =  8,
    kSioDrvStatRtsNotSupported           = 16,
    kSioDrvStatDtrNotSupported           = 32,
    kSioDrvStatClrNotSupported           = 64

} tSioDrvStateErrorCode;


//---------------------------------------------------------------
typedef struct
{
    BYTE  m_bRTS;
    BYTE  m_bDTR;
    BYTE  m_bCTS;
    BYTE  m_bDSR;
    BYTE  m_bDCD;
    BYTE  m_bRI;
    DWORD m_dwSoRxQ;
    DWORD m_dwSoTxQ;
    DWORD m_dwCbRxQ;
    DWORD m_dwCbTxQ;
    WORD  m_wSioStat;
    BYTE  m_bClr;
    BYTE  m_bErrCode;

} tSioState;



typedef struct
{
    BYTE  m_bErrCode;
    DWORD m_dwBaudrate;
    BYTE  m_bDataBits;
    BYTE  m_bParity;
    BYTE  m_bStopBits;
    BYTE  m_bProtocol;

} tSioCfgParam;



//Structure type definition for a serial instance.
typedef struct
{
    BYTE FAR*  m_pbSioState;                            // pointer to driver state

    // Interface functions to upper layer
    BOOL (*m_pfnGetState)(tSioState FAR* pSioState_p);  // Function pointer used to get the state of a specific control line
    BOOL (*m_pfnSetState)(tSioState FAR* pSioState_p);  // Function pointer used to set the state of a specific control line
    BOOL (*m_pfnGetByte) (char FAR* pbValue);           // Function pointer used for reading bytes from RX-FIFO
    BOOL (*m_pfnPutByte) (char      bValue);            // Function pointer used for putting bytes to TX-FIFO

    // Interface functions to lower layer
    BOOL (*m_pfnTxFifoDataSize)(DWORD FAR* pdwSize_p);  // Function pointer used for getting length of data from TX-FIFO
    BOOL (*m_pfnRxFifoDataSize)(DWORD FAR* pdwSize_p);  // Function pointer used for getting length of data from RX-FIFO
    BOOL (*m_pfnSerialInit)    (tSioCfgParam FAR* SioCfgParam_p);
    BOOL (*m_pfnSerialClose)   (void);

} tSerialInst;



void              SioInitInstData (void);
tSerialInst FAR*  SioGetInstData  (BYTE bInstNum_p);


#endif //_SERIAL_H_

// EOF

